/*      */ package org.hibernate.cfg;
/*      */ 
/*      */ import java.lang.annotation.Annotation;
/*      */ import java.util.ArrayList;
/*      */ import java.util.Collections;
/*      */ import java.util.Comparator;
/*      */ import java.util.EnumSet;
/*      */ import java.util.HashMap;
/*      */ import java.util.HashSet;
/*      */ import java.util.Iterator;
/*      */ import java.util.List;
/*      */ import java.util.Map;
/*      */ import java.util.Properties;
/*      */ import java.util.Set;
/*      */ import javax.persistence.Basic;
/*      */ import javax.persistence.DiscriminatorColumn;
/*      */ import javax.persistence.DiscriminatorType;
/*      */ import javax.persistence.DiscriminatorValue;
/*      */ import javax.persistence.Embeddable;
/*      */ import javax.persistence.Embedded;
/*      */ import javax.persistence.EmbeddedId;
/*      */ import javax.persistence.FetchType;
/*      */ import javax.persistence.GeneratedValue;
/*      */ import javax.persistence.GenerationType;
/*      */ import javax.persistence.Id;
/*      */ import javax.persistence.IdClass;
/*      */ import javax.persistence.InheritanceType;
/*      */ import javax.persistence.JoinColumn;
/*      */ import javax.persistence.JoinColumns;
/*      */ import javax.persistence.JoinTable;
/*      */ import javax.persistence.ManyToMany;
/*      */ import javax.persistence.MappedSuperclass;
/*      */ import javax.persistence.OneToMany;
/*      */ import javax.persistence.OneToOne;
/*      */ import javax.persistence.PrimaryKeyJoinColumn;
/*      */ import javax.persistence.PrimaryKeyJoinColumns;
/*      */ import javax.persistence.SecondaryTable;
/*      */ import javax.persistence.SecondaryTables;
/*      */ import javax.persistence.SequenceGenerator;
/*      */ import javax.persistence.SqlResultSetMapping;
/*      */ import javax.persistence.SqlResultSetMappings;
/*      */ import javax.persistence.TableGenerator;
/*      */ import javax.persistence.Transient;
/*      */ import javax.persistence.Version;
/*      */ import org.apache.commons.logging.Log;
/*      */ import org.apache.commons.logging.LogFactory;
/*      */ import org.hibernate.AnnotationException;
/*      */ import org.hibernate.AssertionFailure;
/*      */ import org.hibernate.EntityMode;
/*      */ import org.hibernate.MappingException;
/*      */ import org.hibernate.annotations.AccessType;
/*      */ import org.hibernate.annotations.BatchSize;
/*      */ import org.hibernate.annotations.Cache;
/*      */ import org.hibernate.annotations.Cascade;
/*      */ import org.hibernate.annotations.Check;
/*      */ import org.hibernate.annotations.CollectionId;
/*      */ import org.hibernate.annotations.CollectionOfElements;
/*      */ import org.hibernate.annotations.Columns;
/*      */ import org.hibernate.annotations.DiscriminatorFormula;
/*      */ import org.hibernate.annotations.Fetch;
/*      */ import org.hibernate.annotations.Filter;
/*      */ import org.hibernate.annotations.FilterDef;
/*      */ import org.hibernate.annotations.FilterDefs;
/*      */ import org.hibernate.annotations.Filters;
/*      */ import org.hibernate.annotations.ForeignKey;
/*      */ import org.hibernate.annotations.Formula;
/*      */ import org.hibernate.annotations.GenericGenerator;
/*      */ import org.hibernate.annotations.GenericGenerators;
/*      */ import org.hibernate.annotations.Index;
/*      */ import org.hibernate.annotations.LazyToOne;
/*      */ import org.hibernate.annotations.LazyToOneOption;
/*      */ import org.hibernate.annotations.ManyToAny;
/*      */ import org.hibernate.annotations.MapKeyManyToMany;
/*      */ import org.hibernate.annotations.NaturalId;
/*      */ import org.hibernate.annotations.NotFound;
/*      */ import org.hibernate.annotations.NotFoundAction;
/*      */ import org.hibernate.annotations.OnDelete;
/*      */ import org.hibernate.annotations.OnDeleteAction;
/*      */ import org.hibernate.annotations.ParamDef;
/*      */ import org.hibernate.annotations.Parameter;
/*      */ import org.hibernate.annotations.Parent;
/*      */ import org.hibernate.annotations.Proxy;
/*      */ import org.hibernate.annotations.Sort;
/*      */ import org.hibernate.annotations.Tables;
/*      */ import org.hibernate.annotations.Target;
/*      */ import org.hibernate.annotations.Tuplizer;
/*      */ import org.hibernate.annotations.Tuplizers;
/*      */ import org.hibernate.annotations.TypeDef;
/*      */ import org.hibernate.annotations.TypeDefs;
/*      */ import org.hibernate.annotations.Where;
/*      */ import org.hibernate.annotations.common.reflection.ReflectionManager;
/*      */ import org.hibernate.annotations.common.reflection.XAnnotatedElement;
/*      */ import org.hibernate.annotations.common.reflection.XClass;
/*      */ import org.hibernate.annotations.common.reflection.XPackage;
/*      */ import org.hibernate.annotations.common.reflection.XProperty;
/*      */ import org.hibernate.cfg.annotations.CollectionBinder;
/*      */ import org.hibernate.cfg.annotations.EntityBinder;
/*      */ import org.hibernate.cfg.annotations.Nullability;
/*      */ import org.hibernate.cfg.annotations.PropertyBinder;
/*      */ import org.hibernate.cfg.annotations.QueryBinder;
/*      */ import org.hibernate.cfg.annotations.SimpleValueBinder;
/*      */ import org.hibernate.cfg.annotations.TableBinder;
/*      */ import org.hibernate.engine.FilterDefinition;
/*      */ import org.hibernate.id.MultipleHiLoPerTableGenerator;
/*      */ import org.hibernate.mapping.Component;
/*      */ import org.hibernate.mapping.DependantValue;
/*      */ import org.hibernate.mapping.IdGenerator;
/*      */ import org.hibernate.mapping.Join;
/*      */ import org.hibernate.mapping.JoinedSubclass;
/*      */ import org.hibernate.mapping.KeyValue;
/*      */ import org.hibernate.mapping.PersistentClass;
/*      */ import org.hibernate.mapping.Property;
/*      */ import org.hibernate.mapping.RootClass;
/*      */ import org.hibernate.mapping.SimpleValue;
/*      */ import org.hibernate.mapping.SingleTableSubclass;
/*      */ import org.hibernate.mapping.Subclass;
/*      */ import org.hibernate.mapping.ToOne;
/*      */ import org.hibernate.mapping.UnionSubclass;
/*      */ import org.hibernate.persister.entity.JoinedSubclassEntityPersister;
/*      */ import org.hibernate.persister.entity.SingleTableEntityPersister;
/*      */ import org.hibernate.persister.entity.UnionSubclassEntityPersister;
/*      */ import org.hibernate.type.TypeFactory;
/*      */ import org.hibernate.util.StringHelper;
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ public final class AnnotationBinder
/*      */ {
/*  168 */   private static final Log log = LogFactory.getLog(AnnotationBinder.class);
/*      */   
/*      */   public static void bindDefaults(ExtendedMappings mappings) {
/*  171 */     Map defaults = mappings.getReflectionManager().getDefaults();
/*      */     
/*  173 */     List<SequenceGenerator> anns = (List)defaults.get(SequenceGenerator.class);
/*  174 */     if (anns != null) {
/*  175 */       for (SequenceGenerator ann : anns) {
/*  176 */         IdGenerator idGen = buildIdGenerator(ann, mappings);
/*  177 */         if (idGen != null) { mappings.addDefaultGenerator(idGen);
/*      */         }
/*      */       }
/*      */     }
/*      */     
/*  182 */     List<TableGenerator> anns = (List)defaults.get(TableGenerator.class);
/*  183 */     if (anns != null) {
/*  184 */       for (TableGenerator ann : anns) {
/*  185 */         IdGenerator idGen = buildIdGenerator(ann, mappings);
/*  186 */         if (idGen != null) { mappings.addDefaultGenerator(idGen);
/*      */         }
/*      */       }
/*      */     }
/*      */     
/*  191 */     List<javax.persistence.NamedQuery> anns = (List)defaults.get(javax.persistence.NamedQuery.class);
/*  192 */     if (anns != null) {
/*  193 */       for (javax.persistence.NamedQuery ann : anns) {
/*  194 */         QueryBinder.bindQuery(ann, mappings, true);
/*      */       }
/*      */     }
/*      */     
/*      */ 
/*  199 */     List<javax.persistence.NamedNativeQuery> anns = (List)defaults.get(javax.persistence.NamedNativeQuery.class);
/*  200 */     if (anns != null) {
/*  201 */       for (javax.persistence.NamedNativeQuery ann : anns) {
/*  202 */         QueryBinder.bindNativeQuery(ann, mappings, true);
/*      */       }
/*      */     }
/*      */     
/*      */ 
/*  207 */     List<SqlResultSetMapping> anns = (List)defaults.get(SqlResultSetMapping.class);
/*  208 */     if (anns != null) {
/*  209 */       for (SqlResultSetMapping ann : anns) {
/*  210 */         QueryBinder.bindSqlResultsetMapping(ann, mappings, true);
/*      */       }
/*      */     }
/*      */   }
/*      */   
/*      */   public static void bindPackage(String packageName, ExtendedMappings mappings)
/*      */   {
/*  217 */     XPackage pckg = null;
/*      */     try {
/*  219 */       pckg = mappings.getReflectionManager().packageForName(packageName);
/*      */     }
/*      */     catch (ClassNotFoundException cnf) {
/*  222 */       log.warn("Package not found or wo package-info.java: " + packageName);
/*  223 */       return;
/*      */     }
/*  225 */     if (pckg.isAnnotationPresent(SequenceGenerator.class)) {
/*  226 */       SequenceGenerator ann = (SequenceGenerator)pckg.getAnnotation(SequenceGenerator.class);
/*  227 */       IdGenerator idGen = buildIdGenerator(ann, mappings);
/*  228 */       mappings.addGenerator(idGen);
/*  229 */       log.debug("Add sequence generator with name: " + idGen.getName());
/*      */     }
/*  231 */     if (pckg.isAnnotationPresent(TableGenerator.class)) {
/*  232 */       TableGenerator ann = (TableGenerator)pckg.getAnnotation(TableGenerator.class);
/*  233 */       IdGenerator idGen = buildIdGenerator(ann, mappings);
/*  234 */       mappings.addGenerator(idGen);
/*      */     }
/*      */     
/*  237 */     bindGenericGenerators(pckg, mappings);
/*  238 */     bindQueries(pckg, mappings);
/*  239 */     bindFilterDefs(pckg, mappings);
/*  240 */     bindTypeDefs(pckg, mappings);
/*  241 */     BinderHelper.bindAnyMetaDefs(pckg, mappings);
/*      */   }
/*      */   
/*      */   private static void bindGenericGenerators(XAnnotatedElement annotatedElement, ExtendedMappings mappings) {
/*  245 */     GenericGenerator defAnn = (GenericGenerator)annotatedElement.getAnnotation(GenericGenerator.class);
/*  246 */     GenericGenerators defsAnn = (GenericGenerators)annotatedElement.getAnnotation(GenericGenerators.class);
/*  247 */     if (defAnn != null) {
/*  248 */       bindGenericGenerator(defAnn, mappings);
/*      */     }
/*  250 */     if (defsAnn != null) {
/*  251 */       for (GenericGenerator def : defsAnn.value()) {
/*  252 */         bindGenericGenerator(def, mappings);
/*      */       }
/*      */     }
/*      */   }
/*      */   
/*      */   private static void bindGenericGenerator(GenericGenerator def, ExtendedMappings mappings) {
/*  258 */     IdGenerator idGen = buildIdGenerator(def, mappings);
/*  259 */     mappings.addGenerator(idGen);
/*      */   }
/*      */   
/*      */   private static void bindQueries(XAnnotatedElement annotatedElement, ExtendedMappings mappings)
/*      */   {
/*  264 */     SqlResultSetMapping ann = (SqlResultSetMapping)annotatedElement.getAnnotation(SqlResultSetMapping.class);
/*  265 */     QueryBinder.bindSqlResultsetMapping(ann, mappings, false);
/*      */     
/*      */ 
/*  268 */     SqlResultSetMappings ann = (SqlResultSetMappings)annotatedElement.getAnnotation(SqlResultSetMappings.class);
/*  269 */     if (ann != null) {
/*  270 */       for (SqlResultSetMapping current : ann.value()) {
/*  271 */         QueryBinder.bindSqlResultsetMapping(current, mappings, false);
/*      */       }
/*      */     }
/*      */     
/*      */ 
/*  276 */     javax.persistence.NamedQuery ann = (javax.persistence.NamedQuery)annotatedElement.getAnnotation(javax.persistence.NamedQuery.class);
/*  277 */     QueryBinder.bindQuery(ann, mappings, false);
/*      */     
/*      */ 
/*  280 */     org.hibernate.annotations.NamedQuery ann = (org.hibernate.annotations.NamedQuery)annotatedElement.getAnnotation(org.hibernate.annotations.NamedQuery.class);
/*      */     
/*      */ 
/*  283 */     QueryBinder.bindQuery(ann, mappings);
/*      */     
/*      */ 
/*  286 */     javax.persistence.NamedQueries ann = (javax.persistence.NamedQueries)annotatedElement.getAnnotation(javax.persistence.NamedQueries.class);
/*  287 */     QueryBinder.bindQueries(ann, mappings, false);
/*      */     
/*      */ 
/*  290 */     org.hibernate.annotations.NamedQueries ann = (org.hibernate.annotations.NamedQueries)annotatedElement.getAnnotation(org.hibernate.annotations.NamedQueries.class);
/*      */     
/*      */ 
/*  293 */     QueryBinder.bindQueries(ann, mappings);
/*      */     
/*      */ 
/*  296 */     javax.persistence.NamedNativeQuery ann = (javax.persistence.NamedNativeQuery)annotatedElement.getAnnotation(javax.persistence.NamedNativeQuery.class);
/*  297 */     QueryBinder.bindNativeQuery(ann, mappings, false);
/*      */     
/*      */ 
/*  300 */     org.hibernate.annotations.NamedNativeQuery ann = (org.hibernate.annotations.NamedNativeQuery)annotatedElement.getAnnotation(org.hibernate.annotations.NamedNativeQuery.class);
/*      */     
/*      */ 
/*  303 */     QueryBinder.bindNativeQuery(ann, mappings);
/*      */     
/*      */ 
/*  306 */     javax.persistence.NamedNativeQueries ann = (javax.persistence.NamedNativeQueries)annotatedElement.getAnnotation(javax.persistence.NamedNativeQueries.class);
/*  307 */     QueryBinder.bindNativeQueries(ann, mappings, false);
/*      */     
/*      */ 
/*  310 */     org.hibernate.annotations.NamedNativeQueries ann = (org.hibernate.annotations.NamedNativeQueries)annotatedElement.getAnnotation(org.hibernate.annotations.NamedNativeQueries.class);
/*      */     
/*      */ 
/*  313 */     QueryBinder.bindNativeQueries(ann, mappings);
/*      */   }
/*      */   
/*      */   private static IdGenerator buildIdGenerator(Annotation ann, Mappings mappings)
/*      */   {
/*  318 */     IdGenerator idGen = new IdGenerator();
/*  319 */     if (mappings.getSchemaName() != null) {
/*  320 */       idGen.addParam("schema", mappings.getSchemaName());
/*      */     }
/*  322 */     if (mappings.getCatalogName() != null) {
/*  323 */       idGen.addParam("catalog", mappings.getCatalogName());
/*      */     }
/*  325 */     if (ann == null) {
/*  326 */       idGen = null;
/*      */     }
/*  328 */     else if ((ann instanceof TableGenerator)) {
/*  329 */       TableGenerator tabGen = (TableGenerator)ann;
/*  330 */       idGen.setName(tabGen.name());
/*  331 */       idGen.setIdentifierGeneratorStrategy(MultipleHiLoPerTableGenerator.class.getName());
/*      */       
/*  333 */       if (!BinderHelper.isDefault(tabGen.table())) {
/*  334 */         idGen.addParam("table", tabGen.table());
/*      */       }
/*  336 */       if (!BinderHelper.isDefault(tabGen.catalog())) {
/*  337 */         idGen.addParam("catalog", tabGen.catalog());
/*      */       }
/*  339 */       if (!BinderHelper.isDefault(tabGen.schema())) {
/*  340 */         idGen.addParam("schema", tabGen.schema());
/*      */       }
/*      */       
/*      */ 
/*  344 */       if (!BinderHelper.isDefault(tabGen.pkColumnName())) {
/*  345 */         idGen.addParam("primary_key_column", tabGen.pkColumnName());
/*      */       }
/*  347 */       if (!BinderHelper.isDefault(tabGen.valueColumnName())) {
/*  348 */         idGen.addParam("value_column", tabGen.valueColumnName());
/*      */       }
/*  350 */       if (!BinderHelper.isDefault(tabGen.pkColumnValue())) {
/*  351 */         idGen.addParam("primary_key_value", tabGen.pkColumnValue());
/*      */       }
/*  353 */       idGen.addParam("max_lo", String.valueOf(tabGen.allocationSize() - 1));
/*  354 */       log.debug("Add table generator with name: " + idGen.getName());
/*      */     }
/*  356 */     else if ((ann instanceof SequenceGenerator)) {
/*  357 */       SequenceGenerator seqGen = (SequenceGenerator)ann;
/*  358 */       idGen.setName(seqGen.name());
/*  359 */       idGen.setIdentifierGeneratorStrategy("seqhilo");
/*      */       
/*  361 */       if (!BinderHelper.isDefault(seqGen.sequenceName())) {
/*  362 */         idGen.addParam("sequence", seqGen.sequenceName());
/*      */       }
/*      */       
/*  365 */       if (seqGen.initialValue() != 1) {
/*  366 */         log.warn("Hibernate does not support SequenceGenerator.initialValue()");
/*      */       }
/*      */       
/*      */ 
/*  370 */       idGen.addParam("max_lo", String.valueOf(seqGen.allocationSize() - 1));
/*  371 */       log.debug("Add sequence generator with name: " + idGen.getName());
/*      */     }
/*  373 */     else if ((ann instanceof GenericGenerator)) {
/*  374 */       GenericGenerator genGen = (GenericGenerator)ann;
/*  375 */       idGen.setName(genGen.name());
/*  376 */       idGen.setIdentifierGeneratorStrategy(genGen.strategy());
/*  377 */       Parameter[] params = genGen.parameters();
/*  378 */       for (Parameter parameter : params) {
/*  379 */         idGen.addParam(parameter.name(), parameter.value());
/*      */       }
/*  381 */       log.debug("Add generic generator with name: " + idGen.getName());
/*      */     }
/*      */     else {
/*  384 */       throw new AssertionFailure("Unknown Generator annotation: " + ann);
/*      */     }
/*  386 */     return idGen;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public static void bindClass(XClass clazzToProcess, Map<XClass, InheritanceState> inheritanceStatePerClass, ExtendedMappings mappings)
/*      */     throws MappingException
/*      */   {
/*  397 */     InheritanceState inheritanceState = (InheritanceState)inheritanceStatePerClass.get(clazzToProcess);
/*  398 */     AnnotatedClassType classType = mappings.getClassType(clazzToProcess);
/*  399 */     if ((AnnotatedClassType.EMBEDDABLE_SUPERCLASS.equals(classType)) || (AnnotatedClassType.NONE.equals(classType)) || (AnnotatedClassType.EMBEDDABLE.equals(classType)))
/*      */     {
/*      */ 
/*      */ 
/*  403 */       if ((AnnotatedClassType.NONE.equals(classType)) && (clazzToProcess.isAnnotationPresent(org.hibernate.annotations.Entity.class)))
/*      */       {
/*  405 */         log.warn("Class annotated @org.hibernate.annotations.Entity but not javax.persistence.Entity (most likely a user error): " + clazzToProcess.getName());
/*      */       }
/*      */       
/*  408 */       return;
/*      */     }
/*  410 */     if (!classType.equals(AnnotatedClassType.ENTITY))
/*      */     {
/*  412 */       throw new AnnotationException("Annotated class should have a @javax.persistence.Entity, @javax.persistence.Embeddable or @javax.persistence.EmbeddedSuperclass annotation: " + clazzToProcess.getName());
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*  417 */     XAnnotatedElement annotatedClass = clazzToProcess;
/*  418 */     if (log.isInfoEnabled()) log.info("Binding entity from annotated class: " + clazzToProcess.getName());
/*  419 */     InheritanceState superEntityState = InheritanceState.getSuperEntityInheritanceState(clazzToProcess, inheritanceStatePerClass, mappings.getReflectionManager());
/*      */     
/*      */ 
/*      */ 
/*  423 */     PersistentClass superEntity = superEntityState != null ? mappings.getClass(superEntityState.clazz.getName()) : null;
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*  428 */     if (superEntity == null)
/*      */     {
/*  430 */       if (inheritanceState.hasParents) {
/*  431 */         throw new AssertionFailure("Subclass has to be binded after it's mother class: " + superEntityState.clazz.getName());
/*      */       }
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*  437 */     bindQueries(annotatedClass, mappings);
/*  438 */     bindFilterDefs(annotatedClass, mappings);
/*  439 */     bindTypeDefs(annotatedClass, mappings);
/*  440 */     BinderHelper.bindAnyMetaDefs(annotatedClass, mappings);
/*      */     
/*  442 */     String schema = "";
/*  443 */     String table = "";
/*  444 */     String catalog = "";
/*  445 */     String discrimValue = null;
/*  446 */     List<String[]> uniqueConstraints = new ArrayList();
/*  447 */     Ejb3DiscriminatorColumn discriminatorColumn = null;
/*  448 */     Ejb3JoinColumn[] inheritanceJoinedColumns = null;
/*      */     
/*  450 */     if (annotatedClass.isAnnotationPresent(javax.persistence.Table.class)) {
/*  451 */       javax.persistence.Table tabAnn = (javax.persistence.Table)annotatedClass.getAnnotation(javax.persistence.Table.class);
/*  452 */       table = tabAnn.name();
/*  453 */       schema = tabAnn.schema();
/*  454 */       catalog = tabAnn.catalog();
/*  455 */       uniqueConstraints = TableBinder.buildUniqueConstraints(tabAnn.uniqueConstraints());
/*      */     }
/*  457 */     boolean hasJoinedColumns = (inheritanceState.hasParents) && (InheritanceType.JOINED.equals(inheritanceState.type));
/*      */     
/*  459 */     if (hasJoinedColumns)
/*      */     {
/*  461 */       PrimaryKeyJoinColumns jcsAnn = (PrimaryKeyJoinColumns)annotatedClass.getAnnotation(PrimaryKeyJoinColumns.class);
/*  462 */       boolean explicitInheritanceJoinedColumns = (jcsAnn != null) && (jcsAnn.value().length != 0);
/*  463 */       if (explicitInheritanceJoinedColumns) {
/*  464 */         int nbrOfInhJoinedColumns = jcsAnn.value().length;
/*      */         
/*  466 */         inheritanceJoinedColumns = new Ejb3JoinColumn[nbrOfInhJoinedColumns];
/*  467 */         for (int colIndex = 0; colIndex < nbrOfInhJoinedColumns; colIndex++) {
/*  468 */           PrimaryKeyJoinColumn jcAnn = jcsAnn.value()[colIndex];
/*  469 */           inheritanceJoinedColumns[colIndex] = Ejb3JoinColumn.buildJoinColumn(jcAnn, null, superEntity.getIdentifier(), (Map)null, (PropertyHolder)null, mappings);
/*      */         }
/*      */         
/*      */ 
/*      */       }
/*      */       else
/*      */       {
/*  476 */         PrimaryKeyJoinColumn jcAnn = (PrimaryKeyJoinColumn)annotatedClass.getAnnotation(PrimaryKeyJoinColumn.class);
/*  477 */         inheritanceJoinedColumns = new Ejb3JoinColumn[1];
/*  478 */         inheritanceJoinedColumns[0] = Ejb3JoinColumn.buildJoinColumn(jcAnn, null, superEntity.getIdentifier(), (Map)null, (PropertyHolder)null, mappings);
/*      */       }
/*      */       
/*      */ 
/*      */ 
/*  483 */       log.debug("Subclass joined column(s) created");
/*      */ 
/*      */     }
/*  486 */     else if ((annotatedClass.isAnnotationPresent(PrimaryKeyJoinColumns.class)) || (annotatedClass.isAnnotationPresent(PrimaryKeyJoinColumn.class)))
/*      */     {
/*  488 */       log.warn("Root entity should not hold an PrimaryKeyJoinColum(s), will be ignored");
/*      */     }
/*      */     
/*      */ 
/*  492 */     if (InheritanceType.SINGLE_TABLE.equals(inheritanceState.type)) {
/*  493 */       DiscriminatorColumn discAnn = (DiscriminatorColumn)annotatedClass.getAnnotation(DiscriminatorColumn.class);
/*      */       
/*      */ 
/*  496 */       DiscriminatorType discriminatorType = discAnn != null ? discAnn.discriminatorType() : DiscriminatorType.STRING;
/*      */       
/*      */ 
/*      */ 
/*  500 */       DiscriminatorFormula discFormulaAnn = (DiscriminatorFormula)annotatedClass.getAnnotation(DiscriminatorFormula.class);
/*      */       
/*      */ 
/*  503 */       if (!inheritanceState.hasParents) {
/*  504 */         discriminatorColumn = Ejb3DiscriminatorColumn.buildDiscriminatorColumn(discriminatorType, discAnn, discFormulaAnn, mappings);
/*      */       }
/*      */       
/*      */ 
/*  508 */       if ((discAnn != null) && (inheritanceState.hasParents)) {
/*  509 */         log.warn("Discriminator column has to be defined in the root entity, it will be ignored in subclass: " + clazzToProcess.getName());
/*      */       }
/*      */       
/*      */ 
/*      */ 
/*  514 */       discrimValue = annotatedClass.isAnnotationPresent(DiscriminatorValue.class) ? ((DiscriminatorValue)annotatedClass.getAnnotation(DiscriminatorValue.class)).value() : null;
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */     PersistentClass persistentClass;
/*      */     
/*      */ 
/*  522 */     if (!inheritanceState.hasParents) {
/*  523 */       persistentClass = new RootClass();
/*      */     } else { PersistentClass persistentClass;
/*  525 */       if (InheritanceType.SINGLE_TABLE.equals(inheritanceState.type)) {
/*  526 */         persistentClass = new SingleTableSubclass(superEntity);
/*      */       } else { PersistentClass persistentClass;
/*  528 */         if (InheritanceType.JOINED.equals(inheritanceState.type)) {
/*  529 */           persistentClass = new JoinedSubclass(superEntity);
/*      */         } else { PersistentClass persistentClass;
/*  531 */           if (InheritanceType.TABLE_PER_CLASS.equals(inheritanceState.type)) {
/*  532 */             persistentClass = new UnionSubclass(superEntity);
/*      */           }
/*      */           else
/*  535 */             throw new AssertionFailure("Unknown inheritance type: " + inheritanceState.type); } } }
/*      */     PersistentClass persistentClass;
/*  537 */     Proxy proxyAnn = (Proxy)annotatedClass.getAnnotation(Proxy.class);
/*  538 */     BatchSize sizeAnn = (BatchSize)annotatedClass.getAnnotation(BatchSize.class);
/*  539 */     Where whereAnn = (Where)annotatedClass.getAnnotation(Where.class);
/*  540 */     javax.persistence.Entity entityAnn = (javax.persistence.Entity)annotatedClass.getAnnotation(javax.persistence.Entity.class);
/*  541 */     org.hibernate.annotations.Entity hibEntityAnn = (org.hibernate.annotations.Entity)annotatedClass.getAnnotation(org.hibernate.annotations.Entity.class);
/*      */     
/*      */ 
/*  544 */     Cache cacheAnn = (Cache)annotatedClass.getAnnotation(Cache.class);
/*      */     
/*      */ 
/*  547 */     EntityBinder entityBinder = new EntityBinder(entityAnn, hibEntityAnn, clazzToProcess, persistentClass, mappings);
/*      */     
/*      */ 
/*  550 */     entityBinder.setDiscriminatorValue(discrimValue);
/*  551 */     entityBinder.setBatchSize(sizeAnn);
/*  552 */     entityBinder.setProxy(proxyAnn);
/*  553 */     entityBinder.setWhere(whereAnn);
/*  554 */     entityBinder.setCache(cacheAnn);
/*  555 */     entityBinder.setInheritanceState(inheritanceState);
/*  556 */     Filter filterAnn = (Filter)annotatedClass.getAnnotation(Filter.class);
/*  557 */     if (filterAnn != null) {
/*  558 */       entityBinder.addFilter(filterAnn.name(), filterAnn.condition());
/*      */     }
/*  560 */     Filters filtersAnn = (Filters)annotatedClass.getAnnotation(Filters.class);
/*  561 */     if (filtersAnn != null) {
/*  562 */       for (Filter filter : filtersAnn.value()) {
/*  563 */         entityBinder.addFilter(filter.name(), filter.condition());
/*      */       }
/*      */     }
/*  566 */     entityBinder.bindEntity();
/*      */     
/*  568 */     if (inheritanceState.hasTable()) {
/*  569 */       Check checkAnn = (Check)annotatedClass.getAnnotation(Check.class);
/*  570 */       String constraints = checkAnn == null ? null : checkAnn.constraints();
/*      */       
/*      */ 
/*  573 */       entityBinder.bindTable(schema, catalog, table, uniqueConstraints, constraints, inheritanceState.hasDenormalizedTable() ? superEntity.getTable() : null);
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     }
/*  581 */     else if (annotatedClass.isAnnotationPresent(javax.persistence.Table.class)) {
/*  582 */       log.warn("Illegal use of @Table in a subclass of a SINGLE_TABLE hierarchy: " + clazzToProcess.getName());
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*  590 */     PropertyHolder propertyHolder = PropertyHolderBuilder.buildPropertyHolder(clazzToProcess, persistentClass, entityBinder, mappings);
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*  596 */     SecondaryTable secTabAnn = (SecondaryTable)annotatedClass.getAnnotation(SecondaryTable.class);
/*      */     
/*      */ 
/*  599 */     SecondaryTables secTabsAnn = (SecondaryTables)annotatedClass.getAnnotation(SecondaryTables.class);
/*      */     
/*      */ 
/*  602 */     entityBinder.firstLevelSecondaryTablesBinding(secTabAnn, secTabsAnn);
/*      */     
/*  604 */     OnDelete onDeleteAnn = (OnDelete)annotatedClass.getAnnotation(OnDelete.class);
/*  605 */     boolean onDeleteAppropriate = false;
/*  606 */     if ((InheritanceType.JOINED.equals(inheritanceState.type)) && (inheritanceState.hasParents)) {
/*  607 */       onDeleteAppropriate = true;
/*  608 */       JoinedSubclass jsc = (JoinedSubclass)persistentClass;
/*  609 */       if (persistentClass.getEntityPersisterClass() == null) {
/*  610 */         persistentClass.getRootClass().setEntityPersisterClass(JoinedSubclassEntityPersister.class);
/*      */       }
/*  612 */       SimpleValue key = new DependantValue(jsc.getTable(), jsc.getIdentifier());
/*  613 */       jsc.setKey(key);
/*  614 */       ForeignKey fk = (ForeignKey)annotatedClass.getAnnotation(ForeignKey.class);
/*  615 */       if ((fk != null) && (!BinderHelper.isDefault(fk.name()))) {
/*  616 */         key.setForeignKeyName(fk.name());
/*      */       }
/*  618 */       if (onDeleteAnn != null) {
/*  619 */         key.setCascadeDeleteEnabled(OnDeleteAction.CASCADE.equals(onDeleteAnn.action()));
/*      */       }
/*      */       else {
/*  622 */         key.setCascadeDeleteEnabled(false);
/*      */       }
/*      */       
/*  625 */       SecondPass sp = new JoinedSubclassFkSecondPass(jsc, inheritanceJoinedColumns, key, mappings);
/*  626 */       mappings.addSecondPass(sp);
/*  627 */       mappings.addSecondPass(new CreateKeySecondPass(jsc));
/*      */ 
/*      */     }
/*  630 */     else if (InheritanceType.SINGLE_TABLE.equals(inheritanceState.type)) {
/*  631 */       if (inheritanceState.hasParents) {
/*  632 */         if (persistentClass.getEntityPersisterClass() == null) {
/*  633 */           persistentClass.getRootClass().setEntityPersisterClass(SingleTableEntityPersister.class);
/*      */         }
/*      */         
/*      */       }
/*  637 */       else if ((inheritanceState.hasSons) || (!discriminatorColumn.isImplicit()))
/*      */       {
/*  639 */         bindDiscriminatorToPersistentClass((RootClass)persistentClass, discriminatorColumn, entityBinder.getSecondaryTables(), propertyHolder);
/*      */         
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*  645 */         entityBinder.bindDiscriminatorValue();
/*      */       }
/*      */       
/*      */     }
/*  649 */     else if ((InheritanceType.TABLE_PER_CLASS.equals(inheritanceState.type)) && 
/*  650 */       (inheritanceState.hasParents) && 
/*  651 */       (persistentClass.getEntityPersisterClass() == null)) {
/*  652 */       persistentClass.getRootClass().setEntityPersisterClass(UnionSubclassEntityPersister.class);
/*      */     }
/*      */     
/*      */ 
/*  656 */     if ((onDeleteAnn != null) && (!onDeleteAppropriate)) {
/*  657 */       log.warn("Inapropriate use of @OnDelete on entity, annotation ignored: " + propertyHolder.getEntityName());
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*  663 */     HashMap<String, IdGenerator> classGenerators = buildLocalGenerators(annotatedClass, mappings);
/*      */     
/*      */ 
/*  666 */     List<PropertyData> elements = getElementsToProcess(clazzToProcess, inheritanceStatePerClass, propertyHolder, entityBinder, mappings);
/*      */     
/*      */ 
/*      */ 
/*  670 */     if (elements == null) {
/*  671 */       throw new AnnotationException("No identifier specified for entity: " + propertyHolder.getEntityName());
/*      */     }
/*  673 */     boolean subclassAndSingleTableStrategy = (inheritanceState.type == InheritanceType.SINGLE_TABLE) && (inheritanceState.hasParents);
/*      */     
/*      */ 
/*  676 */     Set<String> idProperties = new HashSet();
/*  677 */     IdClass idClass = null;
/*  678 */     if (!inheritanceState.hasParents)
/*      */     {
/*  680 */       XClass current = inheritanceState.clazz;
/*  681 */       InheritanceState state = inheritanceState;
/*      */       do {
/*  683 */         current = state.clazz;
/*  684 */         if (current.isAnnotationPresent(IdClass.class)) {
/*  685 */           idClass = (IdClass)current.getAnnotation(IdClass.class);
/*  686 */           break;
/*      */         }
/*  688 */         state = InheritanceState.getSuperclassInheritanceState(current, inheritanceStatePerClass, mappings.getReflectionManager());
/*      */ 
/*      */ 
/*      */       }
/*  692 */       while (state != null);
/*      */     }
/*  694 */     if (idClass != null) {
/*  695 */       XClass compositeClass = mappings.getReflectionManager().toXClass(idClass.value());
/*  696 */       boolean isComponent = true;
/*  697 */       boolean propertyAnnotated = entityBinder.isPropertyAnnotated(compositeClass);
/*  698 */       String propertyAccessor = entityBinder.getPropertyAccessor(compositeClass);
/*  699 */       String generatorType = "assigned";
/*  700 */       String generator = "";
/*  701 */       PropertyData inferredData = new PropertyPreloadedData(entityBinder.getPropertyAccessor(), "id", compositeClass);
/*      */       
/*      */ 
/*  704 */       HashMap<String, IdGenerator> localGenerators = new HashMap();
/*  705 */       boolean ignoreIdAnnotations = entityBinder.isIgnoreIdAnnotations();
/*  706 */       entityBinder.setIgnoreIdAnnotations(true);
/*  707 */       bindId(generatorType, generator, inferredData, null, propertyHolder, localGenerators, isComponent, propertyAnnotated, propertyAccessor, entityBinder, null, true, false, mappings);
/*      */       
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*  721 */       inferredData = new PropertyPreloadedData(propertyAccessor, "_identifierMapper", compositeClass);
/*      */       
/*      */ 
/*  724 */       Component mapper = fillComponent(propertyHolder, inferredData, propertyAnnotated, propertyAccessor, false, entityBinder, true, true, false, mappings);
/*      */       
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*  733 */       entityBinder.setIgnoreIdAnnotations(ignoreIdAnnotations);
/*  734 */       persistentClass.setIdentifierMapper(mapper);
/*  735 */       Property property = new Property();
/*  736 */       property.setName("_identifierMapper");
/*  737 */       property.setNodeName("id");
/*  738 */       property.setUpdateable(false);
/*  739 */       property.setInsertable(false);
/*  740 */       property.setValue(mapper);
/*  741 */       property.setPropertyAccessorName("embedded");
/*  742 */       persistentClass.addProperty(property);
/*  743 */       entityBinder.setIgnoreIdAnnotations(true);
/*      */       
/*  745 */       Iterator properties = mapper.getPropertyIterator();
/*  746 */       while (properties.hasNext()) {
/*  747 */         idProperties.add(((Property)properties.next()).getName());
/*      */       }
/*      */     }
/*  750 */     Set<String> missingIdProperties = new HashSet(idProperties);
/*  751 */     for (PropertyData propertyAnnotatedElement : elements) {
/*  752 */       String propertyName = propertyAnnotatedElement.getPropertyName();
/*  753 */       if (!idProperties.contains(propertyName)) {
/*  754 */         processElementAnnotations(propertyHolder, subclassAndSingleTableStrategy ? Nullability.FORCED_NULL : Nullability.NO_CONSTRAINT, propertyAnnotatedElement.getProperty(), propertyAnnotatedElement, classGenerators, entityBinder, false, false, false, mappings);
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */       }
/*      */       else
/*      */       {
/*      */ 
/*      */ 
/*      */ 
/*  765 */         missingIdProperties.remove(propertyName);
/*      */       }
/*      */     }
/*      */     
/*  769 */     if (missingIdProperties.size() != 0) {
/*  770 */       StringBuilder missings = new StringBuilder();
/*  771 */       for (String property : missingIdProperties) {
/*  772 */         missings.append(property).append(", ");
/*      */       }
/*  774 */       throw new AnnotationException("Unable to find properties (" + missings.substring(0, missings.length() - 2) + ") in entity annotated with @IdClass:" + persistentClass.getEntityName());
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*  781 */     if (!inheritanceState.hasParents) {
/*  782 */       RootClass rootClass = (RootClass)persistentClass;
/*  783 */       mappings.addSecondPass(new CreateKeySecondPass(rootClass));
/*      */     }
/*      */     else {
/*  786 */       superEntity.addSubclass((Subclass)persistentClass);
/*      */     }
/*      */     
/*  789 */     mappings.addClass(persistentClass);
/*      */     
/*      */ 
/*  792 */     mappings.addSecondPass(new SecondaryTableSecondPass(entityBinder, propertyHolder, annotatedClass));
/*      */     
/*      */ 
/*  795 */     entityBinder.processComplementaryTableDefinitions((org.hibernate.annotations.Table)annotatedClass.getAnnotation(org.hibernate.annotations.Table.class));
/*  796 */     entityBinder.processComplementaryTableDefinitions((Tables)annotatedClass.getAnnotation(Tables.class));
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private static List<PropertyData> getElementsToProcess(XClass clazzToProcess, Map<XClass, InheritanceState> inheritanceStatePerClass, PropertyHolder propertyHolder, EntityBinder entityBinder, ExtendedMappings mappings)
/*      */   {
/*  809 */     InheritanceState inheritanceState = (InheritanceState)inheritanceStatePerClass.get(clazzToProcess);
/*  810 */     List<XClass> classesToProcess = orderClassesToBeProcessed(clazzToProcess, inheritanceStatePerClass, inheritanceState, mappings);
/*      */     
/*      */ 
/*  813 */     List<PropertyData> elements = new ArrayList();
/*  814 */     int deep = classesToProcess.size();
/*  815 */     boolean hasIdentifier = false;
/*      */     
/*  817 */     assert (!inheritanceState.isEmbeddableSuperclass);
/*  818 */     Boolean isExplicitPropertyAnnotated = null;
/*  819 */     String explicitAccessType = null;
/*  820 */     if (inheritanceState.hasParents) {
/*  821 */       InheritanceState superEntityState = InheritanceState.getSuperEntityInheritanceState(clazzToProcess, inheritanceStatePerClass, mappings.getReflectionManager());
/*      */       
/*      */ 
/*      */ 
/*  825 */       isExplicitPropertyAnnotated = superEntityState != null ? superEntityState.isPropertyAnnotated : null;
/*      */       
/*      */ 
/*  828 */       explicitAccessType = superEntityState != null ? superEntityState.accessType : null;
/*      */ 
/*      */     }
/*      */     else
/*      */     {
/*  833 */       AccessType access = (AccessType)clazzToProcess.getAnnotation(AccessType.class);
/*  834 */       explicitAccessType = access != null ? access.value() : null;
/*      */       
/*      */ 
/*  837 */       if ("property".equals(explicitAccessType)) {
/*  838 */         isExplicitPropertyAnnotated = Boolean.TRUE;
/*      */       }
/*  840 */       else if ("field".equals(explicitAccessType)) {
/*  841 */         isExplicitPropertyAnnotated = Boolean.FALSE;
/*      */       }
/*      */     }
/*  844 */     Boolean isPropertyAnnotated = isExplicitPropertyAnnotated == null ? Boolean.TRUE : isExplicitPropertyAnnotated;
/*      */     
/*      */ 
/*      */ 
/*  848 */     String accessType = explicitAccessType != null ? explicitAccessType : "property";
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*  854 */     AnnotationException exceptionWhileWalkingElements = null;
/*      */     try {
/*  856 */       for (int index = 0; index < deep; index++) {
/*  857 */         XClass clazz = (XClass)classesToProcess.get(index);
/*      */         
/*  859 */         boolean currentHasIdentifier = addElementsOfAClass(elements, propertyHolder, isPropertyAnnotated.booleanValue(), accessType, clazz, mappings);
/*      */         
/*      */ 
/*      */ 
/*  863 */         hasIdentifier = (hasIdentifier) || (currentHasIdentifier);
/*      */       }
/*      */     }
/*      */     catch (AnnotationException e) {
/*  867 */       exceptionWhileWalkingElements = e;
/*      */     }
/*      */     
/*      */ 
/*  871 */     if ((!hasIdentifier) && (!inheritanceState.hasParents)) {
/*  872 */       if (isExplicitPropertyAnnotated != null)
/*      */       {
/*  874 */         if (exceptionWhileWalkingElements != null) throw exceptionWhileWalkingElements;
/*  875 */         return null;
/*      */       }
/*  877 */       isPropertyAnnotated = Boolean.valueOf(!isPropertyAnnotated.booleanValue());
/*  878 */       accessType = "field";
/*  879 */       elements.clear();
/*  880 */       for (int index = 0; index < deep; index++) {
/*  881 */         XClass clazz = (XClass)classesToProcess.get(index);
/*  882 */         boolean currentHasIdentifier = addElementsOfAClass(elements, propertyHolder, isPropertyAnnotated.booleanValue(), accessType, clazz, mappings);
/*      */         
/*      */ 
/*      */ 
/*  886 */         hasIdentifier = (hasIdentifier) || (currentHasIdentifier);
/*      */       }
/*      */     }
/*      */     
/*      */ 
/*  891 */     if ((!hasIdentifier) && (exceptionWhileWalkingElements != null)) { throw exceptionWhileWalkingElements;
/*      */     }
/*      */     
/*  894 */     entityBinder.setPropertyAnnotated(isPropertyAnnotated.booleanValue());
/*  895 */     entityBinder.setPropertyAccessor(accessType);
/*  896 */     inheritanceState.isPropertyAnnotated = isPropertyAnnotated;
/*  897 */     inheritanceState.accessType = accessType;
/*  898 */     return (hasIdentifier) || (inheritanceState.hasParents) ? elements : null;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private static List<XClass> orderClassesToBeProcessed(XClass annotatedClass, Map<XClass, InheritanceState> inheritanceStatePerClass, InheritanceState inheritanceState, ExtendedMappings mappings)
/*      */   {
/*  908 */     List<XClass> classesToProcess = new ArrayList();
/*  909 */     XClass currentClassInHierarchy = annotatedClass;
/*      */     InheritanceState superclassState;
/*      */     do {
/*  912 */       classesToProcess.add(0, currentClassInHierarchy);
/*  913 */       XClass superClass = currentClassInHierarchy;
/*      */       do {
/*  915 */         superClass = superClass.getSuperclass();
/*  916 */         superclassState = (InheritanceState)inheritanceStatePerClass.get(superClass);
/*      */ 
/*      */       }
/*  919 */       while ((superClass != null) && (!mappings.getReflectionManager().equals(superClass, Object.class)) && (superclassState == null));
/*      */       
/*  921 */       currentClassInHierarchy = superClass;
/*      */     }
/*  923 */     while ((superclassState != null) && (superclassState.isEmbeddableSuperclass));
/*      */     
/*  925 */     return classesToProcess;
/*      */   }
/*      */   
/*      */   private static void bindFilterDefs(XAnnotatedElement annotatedElement, ExtendedMappings mappings) {
/*  929 */     FilterDef defAnn = (FilterDef)annotatedElement.getAnnotation(FilterDef.class);
/*  930 */     FilterDefs defsAnn = (FilterDefs)annotatedElement.getAnnotation(FilterDefs.class);
/*  931 */     if (defAnn != null) {
/*  932 */       bindFilterDef(defAnn, mappings);
/*      */     }
/*  934 */     if (defsAnn != null) {
/*  935 */       for (FilterDef def : defsAnn.value()) {
/*  936 */         bindFilterDef(def, mappings);
/*      */       }
/*      */     }
/*      */   }
/*      */   
/*      */   private static void bindFilterDef(FilterDef defAnn, ExtendedMappings mappings) {
/*  942 */     Map<String, org.hibernate.type.Type> params = new HashMap();
/*  943 */     for (ParamDef param : defAnn.parameters()) {
/*  944 */       params.put(param.name(), TypeFactory.heuristicType(param.type()));
/*      */     }
/*  946 */     FilterDefinition def = new FilterDefinition(defAnn.name(), defAnn.defaultCondition(), params);
/*  947 */     if (log.isInfoEnabled()) log.info("Binding filter definition: " + def.getFilterName());
/*  948 */     mappings.addFilterDefinition(def);
/*      */   }
/*      */   
/*      */   private static void bindTypeDefs(XAnnotatedElement annotatedElement, ExtendedMappings mappings) {
/*  952 */     TypeDef defAnn = (TypeDef)annotatedElement.getAnnotation(TypeDef.class);
/*  953 */     TypeDefs defsAnn = (TypeDefs)annotatedElement.getAnnotation(TypeDefs.class);
/*  954 */     if (defAnn != null) {
/*  955 */       bindTypeDef(defAnn, mappings);
/*      */     }
/*  957 */     if (defsAnn != null) {
/*  958 */       for (TypeDef def : defsAnn.value()) {
/*  959 */         bindTypeDef(def, mappings);
/*      */       }
/*      */     }
/*      */   }
/*      */   
/*      */   private static void bindTypeDef(TypeDef defAnn, ExtendedMappings mappings) {
/*  965 */     Properties params = new Properties();
/*  966 */     for (Parameter param : defAnn.parameters()) {
/*  967 */       params.setProperty(param.name(), param.value());
/*      */     }
/*  969 */     if (log.isInfoEnabled()) log.info("Binding type definition: " + defAnn.name());
/*  970 */     mappings.addTypeDef(defAnn.name(), defAnn.typeClass().getName(), params);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */   private static void bindDiscriminatorToPersistentClass(RootClass rootClass, Ejb3DiscriminatorColumn discriminatorColumn, Map<String, Join> secondaryTables, PropertyHolder propertyHolder)
/*      */   {
/*  978 */     if (rootClass.getDiscriminator() == null) {
/*  979 */       if (discriminatorColumn == null) {
/*  980 */         throw new AssertionFailure("discriminator column should have been built");
/*      */       }
/*  982 */       discriminatorColumn.setJoins(secondaryTables);
/*  983 */       discriminatorColumn.setPropertyHolder(propertyHolder);
/*  984 */       SimpleValue discrim = new SimpleValue(rootClass.getTable());
/*  985 */       rootClass.setDiscriminator(discrim);
/*  986 */       discriminatorColumn.linkWithValue(discrim);
/*  987 */       discrim.setTypeName(discriminatorColumn.getDiscriminatorTypeName());
/*  988 */       rootClass.setPolymorphic(true);
/*  989 */       log.debug("Setting discriminator for entity " + rootClass.getEntityName());
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private static boolean addElementsOfAClass(List<PropertyData> elements, PropertyHolder propertyHolder, boolean isPropertyAnnotated, String propertyAccessor, XClass annotatedClass, ExtendedMappings mappings)
/*      */   {
/* 1000 */     boolean hasIdentifier = false;
/* 1001 */     AccessType access = (AccessType)annotatedClass.getAnnotation(AccessType.class);
/* 1002 */     String localPropertyAccessor = access != null ? access.value() : null;
/*      */     
/*      */ 
/* 1005 */     String accessType = null;
/* 1006 */     if (("property".equals(localPropertyAccessor)) || ("field".equals(localPropertyAccessor))) {
/* 1007 */       accessType = localPropertyAccessor;
/*      */     }
/*      */     else {
/* 1010 */       if (localPropertyAccessor == null) {
/* 1011 */         localPropertyAccessor = propertyAccessor;
/*      */       }
/*      */       
/* 1014 */       if (isPropertyAnnotated) {
/* 1015 */         accessType = "property";
/*      */       }
/*      */       else {
/* 1018 */         accessType = "field";
/*      */       }
/*      */     }
/*      */     
/* 1022 */     log.debug("Processing " + propertyHolder.getEntityName() + " " + accessType + " annotation");
/* 1023 */     List<XProperty> properties = annotatedClass.getDeclaredProperties(accessType);
/*      */     
/* 1025 */     Collections.sort(properties, new Comparator() {
/*      */       public int compare(XProperty property1, XProperty property2) {
/* 1027 */         return property1.getName().compareTo(property2.getName());
/*      */       }
/*      */     });
/* 1030 */     for (XProperty p : properties) {
/* 1031 */       if ((!p.isTypeResolved()) && (!discoverTypeWithoutReflection(p)) && (!mustBeSkipped(p, mappings))) {
/* 1032 */         throw new AnnotationException("Property " + StringHelper.qualify(propertyHolder.getEntityName(), p.getName()) + " has an unbound type and no explicit target entity. Resolve this Generic usage issue" + " or set an explicit target attribute (eg @OneToMany(target=) or use an explicit @Type");
/*      */       }
/*      */       
/*      */ 
/*      */ 
/*      */ 
/* 1038 */       boolean currentHasIdentifier = addProperty(p, elements, localPropertyAccessor, mappings);
/* 1039 */       hasIdentifier = (hasIdentifier) || (currentHasIdentifier);
/*      */     }
/* 1041 */     return hasIdentifier;
/*      */   }
/*      */   
/*      */   private static boolean discoverTypeWithoutReflection(XProperty p) {
/* 1045 */     if ((p.isAnnotationPresent(OneToOne.class)) && (!((OneToOne)p.getAnnotation(OneToOne.class)).targetEntity().equals(Void.TYPE)))
/*      */     {
/*      */ 
/* 1048 */       return true;
/*      */     }
/* 1050 */     if ((p.isAnnotationPresent(OneToMany.class)) && (!((OneToMany)p.getAnnotation(OneToMany.class)).targetEntity().equals(Void.TYPE)))
/*      */     {
/*      */ 
/* 1053 */       return true;
/*      */     }
/* 1055 */     if ((p.isAnnotationPresent(javax.persistence.ManyToOne.class)) && (!((javax.persistence.ManyToOne)p.getAnnotation(javax.persistence.ManyToOne.class)).targetEntity().equals(Void.TYPE)))
/*      */     {
/*      */ 
/* 1058 */       return true;
/*      */     }
/* 1060 */     if ((p.isAnnotationPresent(ManyToMany.class)) && (!((ManyToMany)p.getAnnotation(ManyToMany.class)).targetEntity().equals(Void.TYPE)))
/*      */     {
/*      */ 
/* 1063 */       return true;
/*      */     }
/* 1065 */     if (p.isAnnotationPresent(org.hibernate.annotations.Any.class)) {
/* 1066 */       return true;
/*      */     }
/* 1068 */     if (p.isAnnotationPresent(ManyToAny.class)) {
/* 1069 */       if ((!p.isCollection()) && (!p.isArray())) {
/* 1070 */         throw new AnnotationException("@ManyToAny used on a non collection non array property: " + p.getName());
/*      */       }
/* 1072 */       return true;
/*      */     }
/* 1074 */     if (p.isAnnotationPresent(org.hibernate.annotations.Type.class)) {
/* 1075 */       return true;
/*      */     }
/* 1077 */     if (p.isAnnotationPresent(Target.class)) {
/* 1078 */       return true;
/*      */     }
/* 1080 */     return false;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */   private static boolean addProperty(XProperty property, List<PropertyData> annElts, String propertyAccessor, ExtendedMappings mappings)
/*      */   {
/* 1087 */     boolean hasIdentifier = false;
/* 1088 */     PropertyData propertyAnnotatedElement = new PropertyInferredData(property, propertyAccessor, mappings.getReflectionManager());
/*      */     
/*      */ 
/* 1091 */     if (!mustBeSkipped(propertyAnnotatedElement.getProperty(), mappings))
/*      */     {
/*      */ 
/*      */ 
/*      */ 
/* 1096 */       XAnnotatedElement element = propertyAnnotatedElement.getProperty();
/* 1097 */       if ((element.isAnnotationPresent(Id.class)) || (element.isAnnotationPresent(EmbeddedId.class))) {
/* 1098 */         annElts.add(0, propertyAnnotatedElement);
/* 1099 */         hasIdentifier = true;
/*      */       }
/*      */       else {
/* 1102 */         annElts.add(propertyAnnotatedElement);
/* 1103 */         hasIdentifier = false;
/*      */       }
/*      */     }
/* 1106 */     return hasIdentifier;
/*      */   }
/*      */   
/*      */   private static boolean mustBeSkipped(XProperty property, ExtendedMappings mappings)
/*      */   {
/* 1111 */     return (property.isAnnotationPresent(Transient.class)) || ("net.sf.cglib.transform.impl.InterceptFieldCallback".equals(property.getType().getName())) || ("org.hibernate.bytecode.javassist.FieldHandler".equals(property.getType().getName()));
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private static void processElementAnnotations(PropertyHolder propertyHolder, Nullability nullability, XProperty property, PropertyData inferredData, HashMap<String, IdGenerator> classGenerators, EntityBinder entityBinder, boolean isIdentifierMapper, boolean isComponentEmbedded, boolean inSecondPass, ExtendedMappings mappings)
/*      */     throws MappingException
/*      */   {
/* 1131 */     Ejb3Column[] columns = null;
/* 1132 */     Ejb3JoinColumn[] joinColumns = null;
/* 1133 */     if (log.isDebugEnabled()) {
/* 1134 */       log.debug("Processing annotations of " + propertyHolder.getEntityName() + "." + inferredData.getPropertyName());
/*      */     }
/*      */     
/*      */ 
/*      */ 
/* 1139 */     if (property.isAnnotationPresent(Parent.class)) {
/* 1140 */       if (propertyHolder.isComponent()) {
/* 1141 */         propertyHolder.setParentProperty(property.getName());
/*      */       }
/*      */       else {
/* 1144 */         throw new AnnotationException("@Parent cannot be applied outside an embeddable object: " + StringHelper.qualify(propertyHolder.getPath(), property.getName()));
/*      */       }
/*      */       
/*      */ 
/*      */ 
/* 1149 */       return;
/*      */     }
/*      */     
/*      */ 
/*      */ 
/* 1154 */     JoinColumn[] anns = null;
/* 1155 */     if (property.isAnnotationPresent(JoinColumn.class)) {
/* 1156 */       anns = new JoinColumn[] { (JoinColumn)property.getAnnotation(JoinColumn.class) };
/*      */     }
/* 1158 */     else if (property.isAnnotationPresent(JoinColumns.class)) {
/* 1159 */       JoinColumns ann = (JoinColumns)property.getAnnotation(JoinColumns.class);
/* 1160 */       anns = ann.value();
/* 1161 */       int length = anns.length;
/* 1162 */       if (length == 0) {
/* 1163 */         throw new AnnotationException("Cannot bind an empty @JoinColumns");
/*      */       }
/*      */     }
/* 1166 */     if (anns != null) {
/* 1167 */       joinColumns = Ejb3JoinColumn.buildJoinColumns(anns, null, entityBinder.getSecondaryTables(), propertyHolder, inferredData.getPropertyName(), mappings);
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/* 1173 */     if ((property.isAnnotationPresent(javax.persistence.Column.class)) || (property.isAnnotationPresent(Formula.class))) {
/* 1174 */       javax.persistence.Column ann = (javax.persistence.Column)property.getAnnotation(javax.persistence.Column.class);
/* 1175 */       Formula formulaAnn = (Formula)property.getAnnotation(Formula.class);
/* 1176 */       columns = Ejb3Column.buildColumnFromAnnotation(new javax.persistence.Column[] { ann }, formulaAnn, nullability, propertyHolder, inferredData, entityBinder.getSecondaryTables(), mappings);
/*      */ 
/*      */ 
/*      */ 
/*      */     }
/* 1181 */     else if (property.isAnnotationPresent(Columns.class)) {
/* 1182 */       Columns anns = (Columns)property.getAnnotation(Columns.class);
/* 1183 */       columns = Ejb3Column.buildColumnFromAnnotation(anns.columns(), null, nullability, propertyHolder, inferredData, entityBinder.getSecondaryTables(), mappings);
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 1190 */     if ((joinColumns == null) && ((property.isAnnotationPresent(javax.persistence.ManyToOne.class)) || (property.isAnnotationPresent(OneToOne.class))))
/*      */     {
/*      */ 
/*      */ 
/* 1194 */       if (property.isAnnotationPresent(JoinTable.class)) {
/* 1195 */         JoinTable joinTableAnn = (JoinTable)property.getAnnotation(JoinTable.class);
/* 1196 */         joinColumns = Ejb3JoinColumn.buildJoinColumns(joinTableAnn.inverseJoinColumns(), null, entityBinder.getSecondaryTables(), propertyHolder, inferredData.getPropertyName(), mappings);
/*      */         
/*      */ 
/*      */ 
/* 1200 */         if (StringHelper.isEmpty(joinTableAnn.name())) {
/* 1201 */           throw new AnnotationException("JoinTable.name() on a @ToOne association has to be explicit: " + StringHelper.qualify(propertyHolder.getPath(), inferredData.getPropertyName()));
/*      */         }
/*      */         
/*      */ 
/*      */       }
/*      */       else
/*      */       {
/* 1208 */         OneToOne oneToOneAnn = (OneToOne)property.getAnnotation(OneToOne.class);
/* 1209 */         String mappedBy = oneToOneAnn != null ? oneToOneAnn.mappedBy() : null;
/*      */         
/*      */ 
/* 1212 */         joinColumns = Ejb3JoinColumn.buildJoinColumns((JoinColumn[])null, mappedBy, entityBinder.getSecondaryTables(), propertyHolder, inferredData.getPropertyName(), mappings);
/*      */ 
/*      */       }
/*      */       
/*      */ 
/*      */ 
/*      */     }
/* 1219 */     else if ((joinColumns == null) && ((property.isAnnotationPresent(OneToMany.class)) || (property.isAnnotationPresent(CollectionOfElements.class))))
/*      */     {
/*      */ 
/* 1222 */       OneToMany oneToMany = (OneToMany)property.getAnnotation(OneToMany.class);
/* 1223 */       String mappedBy = oneToMany != null ? oneToMany.mappedBy() : "";
/*      */       
/*      */ 
/* 1226 */       joinColumns = Ejb3JoinColumn.buildJoinColumns((JoinColumn[])null, mappedBy, entityBinder.getSecondaryTables(), propertyHolder, inferredData.getPropertyName(), mappings);
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     }
/* 1232 */     else if ((joinColumns == null) && (property.isAnnotationPresent(org.hibernate.annotations.Any.class))) {
/* 1233 */       throw new AnnotationException("@Any requires an explicit @JoinColumn(s): " + StringHelper.qualify(propertyHolder.getPath(), property.getName()));
/*      */     }
/*      */     
/* 1236 */     if ((columns == null) && (!property.isAnnotationPresent(ManyToMany.class)))
/*      */     {
/* 1238 */       columns = Ejb3Column.buildColumnFromAnnotation(null, null, nullability, propertyHolder, inferredData, entityBinder.getSecondaryTables(), mappings);
/*      */     }
/*      */     
/*      */ 
/*      */ 
/* 1243 */     if (nullability == Nullability.FORCED_NOT_NULL)
/*      */     {
/* 1245 */       for (Ejb3Column col : columns) {
/* 1246 */         col.forceNotNull();
/*      */       }
/*      */     }
/*      */     
/* 1250 */     XClass returnedClass = inferredData.getClassOrElement();
/* 1251 */     if ((!entityBinder.isIgnoreIdAnnotations()) && ((property.isAnnotationPresent(Id.class)) || (property.isAnnotationPresent(EmbeddedId.class))))
/*      */     {
/*      */ 
/* 1254 */       if (isIdentifierMapper) {
/* 1255 */         throw new AnnotationException("@IdClass class should not have @Id nor @EmbeddedId properties");
/*      */       }
/*      */       
/*      */ 
/* 1259 */       log.debug(inferredData.getPropertyName() + " is an id");
/*      */       
/* 1261 */       HashMap<String, IdGenerator> localGenerators = (HashMap)classGenerators.clone();
/* 1262 */       localGenerators.putAll(buildLocalGenerators(property, mappings));
/*      */       
/*      */ 
/*      */ 
/* 1266 */       boolean isComponent = (returnedClass.isAnnotationPresent(Embeddable.class)) || (property.isAnnotationPresent(EmbeddedId.class));
/*      */       
/* 1268 */       boolean propertyAnnotated = entityBinder.isPropertyAnnotated(returnedClass);
/* 1269 */       String propertyAccessor = entityBinder.getPropertyAccessor(returnedClass);
/*      */       
/*      */ 
/* 1272 */       GeneratedValue generatedValue = (GeneratedValue)property.getAnnotation(GeneratedValue.class);
/* 1273 */       String generatorType = generatedValue != null ? generatorType(generatedValue.strategy()) : "assigned";
/*      */       
/*      */ 
/* 1276 */       String generator = generatedValue != null ? generatedValue.generator() : "";
/*      */       
/*      */ 
/* 1279 */       if (isComponent) generatorType = "assigned";
/* 1280 */       org.hibernate.annotations.Type typeAnn = (org.hibernate.annotations.Type)property.getAnnotation(org.hibernate.annotations.Type.class);
/* 1281 */       bindId(generatorType, generator, inferredData, columns, propertyHolder, localGenerators, isComponent, propertyAnnotated, propertyAccessor, entityBinder, typeAnn, false, isIdentifierMapper, mappings);
/*      */       
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 1295 */       if (log.isDebugEnabled()) {
/* 1296 */         log.debug("Bind " + (isComponent ? "@EmbeddedId" : "@Id") + " on " + inferredData.getPropertyName());
/*      */ 
/*      */       }
/*      */       
/*      */ 
/*      */ 
/*      */     }
/* 1303 */     else if (property.isAnnotationPresent(Version.class)) {
/* 1304 */       if (isIdentifierMapper) {
/* 1305 */         throw new AnnotationException("@IdClass class should not have @Version property");
/*      */       }
/*      */       
/*      */ 
/* 1309 */       if (!(propertyHolder.getPersistentClass() instanceof RootClass)) {
/* 1310 */         throw new AnnotationException("Unable to define/override @Version on a subclass: " + propertyHolder.getEntityName());
/*      */       }
/*      */       
/*      */ 
/*      */ 
/* 1315 */       if (!propertyHolder.isEntity()) {
/* 1316 */         throw new AnnotationException("Unable to define @Version on an embedded class: " + propertyHolder.getEntityName());
/*      */       }
/*      */       
/*      */ 
/*      */ 
/* 1321 */       log.debug(inferredData.getPropertyName() + " is a version property");
/* 1322 */       RootClass rootClass = (RootClass)propertyHolder.getPersistentClass();
/* 1323 */       PropertyBinder propBinder = new PropertyBinder();
/* 1324 */       propBinder.setName(inferredData.getPropertyName());
/* 1325 */       propBinder.setReturnedClassName(inferredData.getTypeName());
/* 1326 */       propBinder.setLazy(false);
/* 1327 */       propBinder.setPropertyAccessorName(inferredData.getDefaultAccess());
/* 1328 */       propBinder.setColumns(columns);
/* 1329 */       propBinder.setHolder(propertyHolder);
/* 1330 */       propBinder.setProperty(property);
/* 1331 */       propBinder.setReturnedClass(inferredData.getPropertyClass());
/*      */       
/* 1333 */       propBinder.setMappings(mappings);
/* 1334 */       Property prop = propBinder.bind();
/* 1335 */       rootClass.setVersion(prop);
/* 1336 */       SimpleValue simpleValue = (SimpleValue)prop.getValue();
/* 1337 */       if (!simpleValue.isTypeSpecified()) simpleValue.setTypeName("integer");
/* 1338 */       simpleValue.setNullValue("undefined");
/* 1339 */       rootClass.setOptimisticLockMode(0);
/* 1340 */       log.debug("Version name: " + rootClass.getVersion().getName() + ", unsavedValue: " + ((SimpleValue)rootClass.getVersion().getValue()).getNullValue());
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     }
/* 1346 */     else if (property.isAnnotationPresent(javax.persistence.ManyToOne.class)) {
/* 1347 */       javax.persistence.ManyToOne ann = (javax.persistence.ManyToOne)property.getAnnotation(javax.persistence.ManyToOne.class);
/*      */       
/*      */ 
/* 1350 */       if ((property.isAnnotationPresent(javax.persistence.Column.class)) || (property.isAnnotationPresent(Columns.class)))
/*      */       {
/* 1352 */         throw new AnnotationException("@Column(s) not allowed on a @ManyToOne property: " + StringHelper.qualify(propertyHolder.getPath(), inferredData.getPropertyName()));
/*      */       }
/*      */       
/*      */ 
/* 1356 */       Cascade hibernateCascade = (Cascade)property.getAnnotation(Cascade.class);
/* 1357 */       NotFound notFound = (NotFound)property.getAnnotation(NotFound.class);
/* 1358 */       boolean ignoreNotFound = (notFound != null) && (notFound.action().equals(NotFoundAction.IGNORE));
/* 1359 */       OnDelete onDeleteAnn = (OnDelete)property.getAnnotation(OnDelete.class);
/* 1360 */       boolean onDeleteCascade = (onDeleteAnn != null) && (OnDeleteAction.CASCADE.equals(onDeleteAnn.action()));
/* 1361 */       JoinTable assocTable = (JoinTable)property.getAnnotation(JoinTable.class);
/* 1362 */       if (assocTable != null) {
/* 1363 */         Join join = propertyHolder.addJoin(assocTable, false);
/* 1364 */         for (Ejb3JoinColumn joinColumn : joinColumns) {
/* 1365 */           joinColumn.setSecondaryTableName(join.getTable().getName());
/*      */         }
/*      */       }
/* 1368 */       bindManyToOne(getCascadeStrategy(ann.cascade(), hibernateCascade), joinColumns, ann.optional(), ignoreNotFound, onDeleteCascade, mappings.getReflectionManager().toXClass(ann.targetEntity()), propertyHolder, inferredData, false, isIdentifierMapper, inSecondPass, mappings);
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     }
/* 1378 */     else if (property.isAnnotationPresent(OneToOne.class)) {
/* 1379 */       OneToOne ann = (OneToOne)property.getAnnotation(OneToOne.class);
/*      */       
/*      */ 
/* 1382 */       if ((property.isAnnotationPresent(javax.persistence.Column.class)) || (property.isAnnotationPresent(Columns.class)))
/*      */       {
/* 1384 */         throw new AnnotationException("@Column(s) not allowed on a @OneToOne property: " + StringHelper.qualify(propertyHolder.getPath(), inferredData.getPropertyName()));
/*      */       }
/*      */       
/*      */ 
/*      */ 
/* 1389 */       boolean trueOneToOne = (property.isAnnotationPresent(PrimaryKeyJoinColumn.class)) || (property.isAnnotationPresent(PrimaryKeyJoinColumns.class));
/*      */       
/* 1391 */       Cascade hibernateCascade = (Cascade)property.getAnnotation(Cascade.class);
/* 1392 */       NotFound notFound = (NotFound)property.getAnnotation(NotFound.class);
/* 1393 */       boolean ignoreNotFound = (notFound != null) && (notFound.action().equals(NotFoundAction.IGNORE));
/* 1394 */       OnDelete onDeleteAnn = (OnDelete)property.getAnnotation(OnDelete.class);
/* 1395 */       boolean onDeleteCascade = (onDeleteAnn != null) && (OnDeleteAction.CASCADE.equals(onDeleteAnn.action()));
/* 1396 */       JoinTable assocTable = (JoinTable)property.getAnnotation(JoinTable.class);
/* 1397 */       if (assocTable != null) {
/* 1398 */         Join join = propertyHolder.addJoin(assocTable, false);
/* 1399 */         for (Ejb3JoinColumn joinColumn : joinColumns) {
/* 1400 */           joinColumn.setSecondaryTableName(join.getTable().getName());
/*      */         }
/*      */       }
/* 1403 */       bindOneToOne(getCascadeStrategy(ann.cascade(), hibernateCascade), joinColumns, ann.optional(), getFetchMode(ann.fetch()), ignoreNotFound, onDeleteCascade, mappings.getReflectionManager().toXClass(ann.targetEntity()), propertyHolder, inferredData, ann.mappedBy(), trueOneToOne, isIdentifierMapper, inSecondPass, mappings);
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     }
/* 1414 */     else if (property.isAnnotationPresent(org.hibernate.annotations.Any.class))
/*      */     {
/*      */ 
/* 1417 */       if ((property.isAnnotationPresent(javax.persistence.Column.class)) || (property.isAnnotationPresent(Columns.class)))
/*      */       {
/* 1419 */         throw new AnnotationException("@Column(s) not allowed on a @Any property: " + StringHelper.qualify(propertyHolder.getPath(), inferredData.getPropertyName()));
/*      */       }
/*      */       
/*      */ 
/* 1423 */       Cascade hibernateCascade = (Cascade)property.getAnnotation(Cascade.class);
/* 1424 */       NotFound notFound = (NotFound)property.getAnnotation(NotFound.class);
/* 1425 */       boolean ignoreNotFound = (notFound != null) && (notFound.action().equals(NotFoundAction.IGNORE));
/* 1426 */       OnDelete onDeleteAnn = (OnDelete)property.getAnnotation(OnDelete.class);
/* 1427 */       boolean onDeleteCascade = (onDeleteAnn != null) && (OnDeleteAction.CASCADE.equals(onDeleteAnn.action()));
/* 1428 */       JoinTable assocTable = (JoinTable)property.getAnnotation(JoinTable.class);
/* 1429 */       if (assocTable != null) {
/* 1430 */         Join join = propertyHolder.addJoin(assocTable, false);
/* 1431 */         for (Ejb3JoinColumn joinColumn : joinColumns) {
/* 1432 */           joinColumn.setSecondaryTableName(join.getTable().getName());
/*      */         }
/*      */       }
/* 1435 */       bindAny(getCascadeStrategy(null, hibernateCascade), joinColumns, onDeleteCascade, nullability, propertyHolder, inferredData, entityBinder, isIdentifierMapper, mappings);
/*      */ 
/*      */ 
/*      */ 
/*      */     }
/* 1440 */     else if ((property.isAnnotationPresent(OneToMany.class)) || (property.isAnnotationPresent(ManyToMany.class)) || (property.isAnnotationPresent(CollectionOfElements.class)) || (property.isAnnotationPresent(ManyToAny.class)))
/*      */     {
/*      */ 
/*      */ 
/* 1444 */       OneToMany oneToManyAnn = (OneToMany)property.getAnnotation(OneToMany.class);
/* 1445 */       ManyToMany manyToManyAnn = (ManyToMany)property.getAnnotation(ManyToMany.class);
/* 1446 */       CollectionOfElements collectionOfElementsAnn = (CollectionOfElements)property.getAnnotation(CollectionOfElements.class);
/* 1447 */       org.hibernate.annotations.IndexColumn indexAnn = (org.hibernate.annotations.IndexColumn)property.getAnnotation(org.hibernate.annotations.IndexColumn.class);
/*      */       
/*      */ 
/* 1450 */       JoinTable assocTable = (JoinTable)property.getAnnotation(JoinTable.class);
/*      */       
/* 1452 */       IndexColumn indexColumn = IndexColumn.buildColumnFromAnnotation(indexAnn, propertyHolder, inferredData, mappings);
/*      */       
/*      */ 
/* 1455 */       CollectionBinder collectionBinder = CollectionBinder.getCollectionBinder(propertyHolder.getEntityName(), property, !indexColumn.isImplicit());
/*      */       
/*      */ 
/*      */ 
/*      */ 
/* 1460 */       collectionBinder.setIndexColumn(indexColumn);
/* 1461 */       javax.persistence.MapKey mapKeyAnn = (javax.persistence.MapKey)property.getAnnotation(javax.persistence.MapKey.class);
/* 1462 */       collectionBinder.setMapKey(mapKeyAnn);
/* 1463 */       collectionBinder.setPropertyName(inferredData.getPropertyName());
/* 1464 */       BatchSize batchAnn = (BatchSize)property.getAnnotation(BatchSize.class);
/* 1465 */       collectionBinder.setBatchSize(batchAnn);
/* 1466 */       javax.persistence.OrderBy ejb3OrderByAnn = (javax.persistence.OrderBy)property.getAnnotation(javax.persistence.OrderBy.class);
/* 1467 */       org.hibernate.annotations.OrderBy orderByAnn = (org.hibernate.annotations.OrderBy)property.getAnnotation(org.hibernate.annotations.OrderBy.class);
/* 1468 */       collectionBinder.setEjb3OrderBy(ejb3OrderByAnn);
/* 1469 */       collectionBinder.setSqlOrderBy(orderByAnn);
/* 1470 */       Sort sortAnn = (Sort)property.getAnnotation(Sort.class);
/* 1471 */       collectionBinder.setSort(sortAnn);
/* 1472 */       Cache cachAnn = (Cache)property.getAnnotation(Cache.class);
/* 1473 */       collectionBinder.setCache(cachAnn);
/* 1474 */       collectionBinder.setPropertyHolder(propertyHolder);
/* 1475 */       Cascade hibernateCascade = (Cascade)property.getAnnotation(Cascade.class);
/* 1476 */       NotFound notFound = (NotFound)property.getAnnotation(NotFound.class);
/* 1477 */       boolean ignoreNotFound = (notFound != null) && (notFound.action().equals(NotFoundAction.IGNORE));
/* 1478 */       collectionBinder.setIgnoreNotFound(ignoreNotFound);
/* 1479 */       collectionBinder.setCollectionType(inferredData.getProperty().getElementClass());
/* 1480 */       collectionBinder.setMappings(mappings);
/* 1481 */       collectionBinder.setPropertyAccessorName(inferredData.getDefaultAccess());
/*      */       
/* 1483 */       Ejb3Column[] elementColumns = null;
/* 1484 */       PropertyData virtualProperty = new WrappedInferredData(inferredData, "element");
/* 1485 */       if ((property.isAnnotationPresent(javax.persistence.Column.class)) || (property.isAnnotationPresent(Formula.class)))
/*      */       {
/*      */ 
/* 1488 */         javax.persistence.Column ann = (javax.persistence.Column)property.getAnnotation(javax.persistence.Column.class);
/* 1489 */         Formula formulaAnn = (Formula)property.getAnnotation(Formula.class);
/* 1490 */         elementColumns = Ejb3Column.buildColumnFromAnnotation(new javax.persistence.Column[] { ann }, formulaAnn, nullability, propertyHolder, virtualProperty, entityBinder.getSecondaryTables(), mappings);
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */       }
/* 1500 */       else if (property.isAnnotationPresent(Columns.class)) {
/* 1501 */         Columns anns = (Columns)property.getAnnotation(Columns.class);
/* 1502 */         elementColumns = Ejb3Column.buildColumnFromAnnotation(anns.columns(), null, nullability, propertyHolder, virtualProperty, entityBinder.getSecondaryTables(), mappings);
/*      */ 
/*      */       }
/*      */       else
/*      */       {
/*      */ 
/* 1508 */         elementColumns = Ejb3Column.buildColumnFromAnnotation(null, null, nullability, propertyHolder, virtualProperty, entityBinder.getSecondaryTables(), mappings);
/*      */       }
/*      */       
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 1519 */       org.hibernate.annotations.MapKey hibMapKeyAnn = (org.hibernate.annotations.MapKey)property.getAnnotation(org.hibernate.annotations.MapKey.class);
/*      */       
/*      */ 
/* 1522 */       PropertyData mapKeyVirtualProperty = new WrappedInferredData(inferredData, "mapkey");
/* 1523 */       Ejb3Column[] mapColumns = Ejb3Column.buildColumnFromAnnotation((hibMapKeyAnn != null) && (hibMapKeyAnn.columns().length > 0) ? hibMapKeyAnn.columns() : null, null, Nullability.FORCED_NOT_NULL, propertyHolder, mapKeyVirtualProperty, entityBinder.getSecondaryTables(), mappings);
/*      */       
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 1534 */       collectionBinder.setMapKeyColumns(mapColumns);
/*      */       
/* 1536 */       MapKeyManyToMany mapKeyManyToMany = (MapKeyManyToMany)property.getAnnotation(MapKeyManyToMany.class);
/* 1537 */       Ejb3JoinColumn[] mapJoinColumns = Ejb3JoinColumn.buildJoinColumns(mapKeyManyToMany != null ? mapKeyManyToMany.joinColumns() : null, null, entityBinder.getSecondaryTables(), propertyHolder, mapKeyVirtualProperty.getPropertyName(), mappings);
/*      */       
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 1544 */       collectionBinder.setMapKeyManyToManyColumns(mapJoinColumns);
/*      */       
/*      */ 
/* 1547 */       collectionBinder.setEmbedded(property.isAnnotationPresent(Embedded.class));
/* 1548 */       collectionBinder.setElementColumns(elementColumns);
/* 1549 */       collectionBinder.setProperty(property);
/*      */       
/*      */ 
/* 1552 */       if ((oneToManyAnn != null) && (manyToManyAnn != null)) {
/* 1553 */         throw new AnnotationException("@OneToMany and @ManyToMany on the same property is not allowed: " + propertyHolder.getEntityName() + "." + inferredData.getPropertyName());
/*      */       }
/*      */       
/*      */ 
/*      */ 
/* 1558 */       String mappedBy = null;
/* 1559 */       if (oneToManyAnn != null) {
/* 1560 */         for (Ejb3JoinColumn column : joinColumns) {
/* 1561 */           if (column.isSecondary()) {
/* 1562 */             throw new NotYetImplementedException("Collections having FK in secondary table");
/*      */           }
/*      */         }
/* 1565 */         collectionBinder.setFkJoinColumns(joinColumns);
/* 1566 */         mappedBy = oneToManyAnn.mappedBy();
/* 1567 */         collectionBinder.setTargetEntity(mappings.getReflectionManager().toXClass(oneToManyAnn.targetEntity()));
/*      */         
/*      */ 
/* 1570 */         collectionBinder.setCascadeStrategy(getCascadeStrategy(oneToManyAnn.cascade(), hibernateCascade));
/* 1571 */         collectionBinder.setOneToMany(true);
/*      */       }
/* 1573 */       else if (collectionOfElementsAnn != null) {
/* 1574 */         for (Ejb3JoinColumn column : joinColumns) {
/* 1575 */           if (column.isSecondary()) {
/* 1576 */             throw new NotYetImplementedException("Collections having FK in secondary table");
/*      */           }
/*      */         }
/* 1579 */         collectionBinder.setFkJoinColumns(joinColumns);
/* 1580 */         mappedBy = "";
/* 1581 */         collectionBinder.setTargetEntity(mappings.getReflectionManager().toXClass(collectionOfElementsAnn.targetElement()));
/*      */         
/*      */ 
/*      */ 
/* 1585 */         collectionBinder.setOneToMany(true);
/*      */       }
/* 1587 */       else if (manyToManyAnn != null) {
/* 1588 */         mappedBy = manyToManyAnn.mappedBy();
/* 1589 */         collectionBinder.setTargetEntity(mappings.getReflectionManager().toXClass(manyToManyAnn.targetEntity()));
/*      */         
/*      */ 
/* 1592 */         collectionBinder.setCascadeStrategy(getCascadeStrategy(manyToManyAnn.cascade(), hibernateCascade));
/* 1593 */         collectionBinder.setOneToMany(false);
/*      */       }
/* 1595 */       else if (property.isAnnotationPresent(ManyToAny.class)) {
/* 1596 */         mappedBy = "";
/* 1597 */         collectionBinder.setTargetEntity(mappings.getReflectionManager().toXClass(Void.TYPE));
/*      */         
/*      */ 
/* 1600 */         collectionBinder.setCascadeStrategy(getCascadeStrategy(null, hibernateCascade));
/* 1601 */         collectionBinder.setOneToMany(false);
/*      */       }
/* 1603 */       collectionBinder.setMappedBy(mappedBy);
/* 1604 */       bindJoinedTableAssociation(assocTable, mappings, entityBinder, collectionBinder, propertyHolder, inferredData, mappedBy);
/*      */       
/*      */ 
/*      */ 
/* 1608 */       OnDelete onDeleteAnn = (OnDelete)property.getAnnotation(OnDelete.class);
/* 1609 */       boolean onDeleteCascade = (onDeleteAnn != null) && (OnDeleteAction.CASCADE.equals(onDeleteAnn.action()));
/* 1610 */       collectionBinder.setCascadeDeleteEnabled(onDeleteCascade);
/* 1611 */       if (isIdentifierMapper) {
/* 1612 */         collectionBinder.setInsertable(false);
/* 1613 */         collectionBinder.setUpdatable(false);
/*      */       }
/* 1615 */       if (property.isAnnotationPresent(CollectionId.class)) {
/* 1616 */         HashMap<String, IdGenerator> localGenerators = (HashMap)classGenerators.clone();
/* 1617 */         localGenerators.putAll(buildLocalGenerators(property, mappings));
/* 1618 */         collectionBinder.setLocalGenerators(localGenerators);
/*      */       }
/*      */       
/* 1621 */       collectionBinder.bind();
/*      */ 
/*      */     }
/*      */     else
/*      */     {
/* 1626 */       boolean isComponent = false;
/* 1627 */       Embeddable embeddableAnn = (Embeddable)returnedClass.getAnnotation(Embeddable.class);
/* 1628 */       Embedded embeddedAnn = (Embedded)property.getAnnotation(Embedded.class);
/* 1629 */       isComponent = (embeddedAnn != null) || (embeddableAnn != null);
/*      */       
/* 1631 */       if (isComponent)
/*      */       {
/*      */ 
/*      */ 
/* 1635 */         boolean propertyAnnotated = entityBinder.isPropertyAnnotated(property);
/* 1636 */         String propertyAccessor = entityBinder.getPropertyAccessor(property);
/* 1637 */         bindComponent(inferredData, propertyHolder, propertyAnnotated, propertyAccessor, entityBinder, isIdentifierMapper, mappings, isComponentEmbedded);
/*      */ 
/*      */ 
/*      */       }
/*      */       else
/*      */       {
/*      */ 
/*      */ 
/* 1645 */         boolean optional = true;
/* 1646 */         boolean lazy = false;
/* 1647 */         if (property.isAnnotationPresent(Basic.class)) {
/* 1648 */           Basic ann = (Basic)property.getAnnotation(Basic.class);
/* 1649 */           optional = ann.optional();
/* 1650 */           lazy = ann.fetch() == FetchType.LAZY;
/*      */         }
/*      */         
/* 1653 */         if ((!optional) && (nullability != Nullability.FORCED_NULL))
/*      */         {
/* 1655 */           for (Ejb3Column col : columns) {
/* 1656 */             col.forceNotNull();
/*      */           }
/*      */         }
/*      */         
/* 1660 */         PropertyBinder propBinder = new PropertyBinder();
/* 1661 */         propBinder.setName(inferredData.getPropertyName());
/* 1662 */         propBinder.setReturnedClassName(inferredData.getTypeName());
/* 1663 */         propBinder.setLazy(lazy);
/* 1664 */         propBinder.setPropertyAccessorName(inferredData.getDefaultAccess());
/* 1665 */         propBinder.setColumns(columns);
/* 1666 */         propBinder.setHolder(propertyHolder);
/* 1667 */         propBinder.setProperty(property);
/* 1668 */         propBinder.setReturnedClass(inferredData.getPropertyClass());
/* 1669 */         propBinder.setMappings(mappings);
/* 1670 */         if (isIdentifierMapper) {
/* 1671 */           propBinder.setInsertable(false);
/* 1672 */           propBinder.setUpdatable(false);
/*      */         }
/* 1674 */         propBinder.bind();
/*      */       }
/*      */     }
/*      */     
/*      */ 
/* 1679 */     Index index = (Index)property.getAnnotation(Index.class);
/* 1680 */     if (index != null) {
/* 1681 */       if (joinColumns != null)
/*      */       {
/* 1683 */         for (Ejb3Column column : joinColumns) {
/* 1684 */           column.addIndex(index, inSecondPass);
/*      */         }
/*      */         
/*      */       }
/* 1688 */       else if (columns != null) {
/* 1689 */         for (Ejb3Column column : columns) {
/* 1690 */           column.addIndex(index, inSecondPass);
/*      */         }
/*      */       }
/*      */     }
/*      */     
/*      */ 
/* 1696 */     NaturalId naturalIdAnn = (NaturalId)property.getAnnotation(NaturalId.class);
/* 1697 */     if (naturalIdAnn != null) {
/* 1698 */       if (joinColumns != null) {
/* 1699 */         for (Ejb3Column column : joinColumns) {
/* 1700 */           column.addUniqueKey("_UniqueKey", inSecondPass);
/*      */         }
/*      */         
/*      */       } else {
/* 1704 */         for (Ejb3Column column : columns) {
/* 1705 */           column.addUniqueKey("_UniqueKey", inSecondPass);
/*      */         }
/*      */       }
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private static void bindJoinedTableAssociation(JoinTable joinTableAnn, ExtendedMappings mappings, EntityBinder entityBinder, CollectionBinder collectionBinder, PropertyHolder propertyHolder, PropertyData inferredData, String mappedBy)
/*      */   {
/* 1717 */     TableBinder associationTableBinder = new TableBinder();
/*      */     JoinColumn[] annInverseJoins;
/*      */     JoinColumn[] annJoins;
/* 1720 */     JoinColumn[] annInverseJoins; if (joinTableAnn != null) {
/* 1721 */       collectionBinder.setExplicitAssociationTable(true);
/* 1722 */       if (!BinderHelper.isDefault(joinTableAnn.schema()))
/* 1723 */         associationTableBinder.setSchema(joinTableAnn.schema());
/* 1724 */       if (!BinderHelper.isDefault(joinTableAnn.catalog()))
/* 1725 */         associationTableBinder.setCatalog(joinTableAnn.catalog());
/* 1726 */       if (!BinderHelper.isDefault(joinTableAnn.name())) associationTableBinder.setName(joinTableAnn.name());
/* 1727 */       associationTableBinder.setUniqueConstraints(joinTableAnn.uniqueConstraints());
/*      */       
/*      */ 
/*      */ 
/* 1731 */       JoinColumn[] joins = joinTableAnn.joinColumns();
/*      */       JoinColumn[] annJoins;
/* 1733 */       JoinColumn[] annJoins; if (joins.length == 0) {
/* 1734 */         annJoins = null;
/*      */       }
/*      */       else {
/* 1737 */         annJoins = joins;
/*      */       }
/*      */       
/* 1740 */       JoinColumn[] inverseJoins = joinTableAnn.inverseJoinColumns();
/*      */       JoinColumn[] annInverseJoins;
/* 1742 */       if (inverseJoins.length == 0) {
/* 1743 */         annInverseJoins = null;
/*      */       }
/*      */       else {
/* 1746 */         annInverseJoins = inverseJoins;
/*      */       }
/*      */     }
/*      */     else {
/* 1750 */       annJoins = null;
/* 1751 */       annInverseJoins = null;
/*      */     }
/* 1753 */     Ejb3JoinColumn[] joinColumns = Ejb3JoinColumn.buildJoinTableJoinColumns(annJoins, entityBinder.getSecondaryTables(), propertyHolder, inferredData.getPropertyName(), mappedBy, mappings);
/*      */     
/*      */ 
/*      */ 
/* 1757 */     Ejb3JoinColumn[] inverseJoinColumns = Ejb3JoinColumn.buildJoinTableJoinColumns(annInverseJoins, entityBinder.getSecondaryTables(), propertyHolder, inferredData.getPropertyName(), mappedBy, mappings);
/*      */     
/*      */ 
/*      */ 
/* 1761 */     associationTableBinder.setMappings(mappings);
/* 1762 */     collectionBinder.setTableBinder(associationTableBinder);
/* 1763 */     collectionBinder.setJoinColumns(joinColumns);
/* 1764 */     collectionBinder.setInverseJoinColumns(inverseJoinColumns);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private static void bindComponent(PropertyData inferredData, PropertyHolder propertyHolder, boolean propertyAnnotated, String propertyAccessor, EntityBinder entityBinder, boolean isIdentifierMapper, ExtendedMappings mappings, boolean isComponentEmbedded)
/*      */   {
/* 1775 */     Component comp = fillComponent(propertyHolder, inferredData, propertyAnnotated, propertyAccessor, true, entityBinder, isComponentEmbedded, isIdentifierMapper, false, mappings);
/*      */     
/*      */ 
/*      */ 
/*      */ 
/* 1780 */     XProperty property = inferredData.getProperty();
/* 1781 */     setupComponentTuplizer(property, comp);
/*      */     
/* 1783 */     PropertyBinder binder = new PropertyBinder();
/* 1784 */     binder.setName(inferredData.getPropertyName());
/* 1785 */     binder.setValue(comp);
/* 1786 */     binder.setProperty(inferredData.getProperty());
/* 1787 */     binder.setPropertyAccessorName(inferredData.getDefaultAccess());
/* 1788 */     Property prop = binder.make();
/* 1789 */     propertyHolder.addProperty(prop);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public static Component fillComponent(PropertyHolder propertyHolder, PropertyData inferredData, boolean propertyAnnotated, String propertyAccessor, boolean isNullable, EntityBinder entityBinder, boolean isComponentEmbedded, boolean isIdentifierMapper, boolean inSecondPass, ExtendedMappings mappings)
/*      */   {
/* 1803 */     Component comp = new Component(propertyHolder.getPersistentClass());
/* 1804 */     comp.setEmbedded(isComponentEmbedded);
/*      */     
/* 1806 */     comp.setTable(propertyHolder.getTable());
/* 1807 */     if (!isIdentifierMapper) {
/* 1808 */       comp.setComponentClassName(inferredData.getClassOrElementName());
/*      */     }
/*      */     else {
/* 1811 */       comp.setComponentClassName(comp.getOwner().getClassName());
/*      */     }
/* 1813 */     comp.setNodeName(inferredData.getPropertyName());
/* 1814 */     String subpath = StringHelper.qualify(propertyHolder.getPath(), inferredData.getPropertyName());
/* 1815 */     log.debug("Binding component with path: " + subpath);
/* 1816 */     PropertyHolder subHolder = PropertyHolderBuilder.buildPropertyHolder(comp, subpath, inferredData, propertyHolder, mappings);
/*      */     
/*      */ 
/*      */ 
/* 1820 */     List<PropertyData> classElements = new ArrayList();
/* 1821 */     XClass returnedClassOrElement = inferredData.getClassOrElement();
/* 1822 */     addElementsOfAClass(classElements, subHolder, propertyAnnotated, propertyAccessor, returnedClassOrElement, mappings);
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 1829 */     XClass superClass = inferredData.getPropertyClass().getSuperclass();
/* 1830 */     while ((superClass != null) && (superClass.isAnnotationPresent(MappedSuperclass.class)))
/*      */     {
/* 1832 */       addElementsOfAClass(classElements, subHolder, entityBinder.isPropertyAnnotated(superClass), propertyAccessor, superClass, mappings);
/*      */       
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 1838 */       superClass = superClass.getSuperclass();
/*      */     }
/* 1840 */     for (PropertyData propertyAnnotatedElement : classElements) {
/* 1841 */       processElementAnnotations(subHolder, isNullable ? Nullability.NO_CONSTRAINT : Nullability.FORCED_NOT_NULL, propertyAnnotatedElement.getProperty(), propertyAnnotatedElement, new HashMap(), entityBinder, isIdentifierMapper, isComponentEmbedded, inSecondPass, mappings);
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 1850 */     return comp;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private static void bindId(String generatorType, String generatorName, PropertyData inferredData, Ejb3Column[] columns, PropertyHolder propertyHolder, Map<String, IdGenerator> localGenerators, boolean isComposite, boolean isPropertyAnnotated, String propertyAccessor, EntityBinder entityBinder, org.hibernate.annotations.Type typeAnn, boolean isEmbedded, boolean isIdentifierMapper, ExtendedMappings mappings)
/*      */   {
/* 1865 */     PersistentClass persistentClass = propertyHolder.getPersistentClass();
/* 1866 */     if (!(persistentClass instanceof RootClass)) {
/* 1867 */       throw new AnnotationException("Unable to define/override @Id(s) on a subclass: " + propertyHolder.getEntityName());
/*      */     }
/*      */     
/*      */ 
/*      */ 
/* 1872 */     RootClass rootClass = (RootClass)persistentClass;
/* 1873 */     String persistentClassName = rootClass == null ? null : rootClass.getClassName();
/*      */     
/*      */     SimpleValue id;
/*      */     
/* 1877 */     if (isComposite) {
/* 1878 */       SimpleValue id = fillComponent(propertyHolder, inferredData, isPropertyAnnotated, propertyAccessor, false, entityBinder, isEmbedded, isIdentifierMapper, false, mappings);
/*      */       
/*      */ 
/*      */ 
/* 1882 */       Component componentId = (Component)id;
/* 1883 */       componentId.setKey(true);
/* 1884 */       if (rootClass.getIdentifier() != null) {
/* 1885 */         throw new AnnotationException(componentId.getComponentClassName() + " must not have @Id properties when used as an @EmbeddedId");
/*      */       }
/* 1887 */       if (componentId.getPropertySpan() == 0) {
/* 1888 */         throw new AnnotationException(componentId.getComponentClassName() + " has no persistent id property");
/*      */       }
/*      */       
/* 1891 */       XProperty property = inferredData.getProperty();
/* 1892 */       setupComponentTuplizer(property, componentId);
/*      */     }
/*      */     else {
/* 1895 */       for (Ejb3Column column : columns) {
/* 1896 */         column.forceNotNull();
/*      */       }
/* 1898 */       SimpleValueBinder value = new SimpleValueBinder();
/* 1899 */       value.setPropertyName(inferredData.getPropertyName());
/* 1900 */       value.setReturnedClassName(inferredData.getTypeName());
/* 1901 */       value.setColumns(columns);
/* 1902 */       value.setPersistentClassName(persistentClassName);
/* 1903 */       value.setMappings(mappings);
/* 1904 */       value.setExplicitType(typeAnn);
/* 1905 */       id = value.make();
/*      */     }
/* 1907 */     rootClass.setIdentifier(id);
/* 1908 */     BinderHelper.makeIdGenerator(id, generatorType, generatorName, mappings, localGenerators);
/* 1909 */     if (isEmbedded) {
/* 1910 */       rootClass.setEmbeddedIdentifier(inferredData.getPropertyClass() == null);
/*      */     }
/*      */     else {
/* 1913 */       PropertyBinder binder = new PropertyBinder();
/* 1914 */       binder.setName(inferredData.getPropertyName());
/* 1915 */       binder.setValue(id);
/* 1916 */       binder.setPropertyAccessorName(inferredData.getDefaultAccess());
/* 1917 */       binder.setProperty(inferredData.getProperty());
/* 1918 */       Property prop = binder.make();
/* 1919 */       rootClass.setIdentifierProperty(prop);
/*      */     }
/*      */   }
/*      */   
/*      */   private static void setupComponentTuplizer(XProperty property, Component component) {
/* 1924 */     if (property == null) return;
/* 1925 */     if (property.isAnnotationPresent(Tuplizers.class)) {
/* 1926 */       for (Tuplizer tuplizer : ((Tuplizers)property.getAnnotation(Tuplizers.class)).value()) {
/* 1927 */         EntityMode mode = EntityMode.parse(tuplizer.entityMode());
/* 1928 */         component.addTuplizer(mode, tuplizer.impl().getName());
/*      */       }
/*      */     }
/* 1931 */     if (property.isAnnotationPresent(Tuplizer.class)) {
/* 1932 */       Tuplizer tuplizer = (Tuplizer)property.getAnnotation(Tuplizer.class);
/* 1933 */       EntityMode mode = EntityMode.parse(tuplizer.entityMode());
/* 1934 */       component.addTuplizer(mode, tuplizer.impl().getName());
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private static void bindManyToOne(String cascadeStrategy, Ejb3JoinColumn[] columns, boolean optional, boolean ignoreNotFound, boolean cascadeOnDelete, XClass targetEntity, PropertyHolder propertyHolder, PropertyData inferredData, boolean unique, boolean isIdentifierMapper, boolean inSecondPass, ExtendedMappings mappings)
/*      */   {
/* 1946 */     org.hibernate.mapping.ManyToOne value = new org.hibernate.mapping.ManyToOne(columns[0].getTable());
/* 1947 */     if (isDefault(targetEntity, mappings)) {
/* 1948 */       value.setReferencedEntityName(inferredData.getClassOrElementName());
/*      */     }
/*      */     else {
/* 1951 */       value.setReferencedEntityName(targetEntity.getName());
/*      */     }
/* 1953 */     defineFetchingStrategy(value, inferredData.getProperty());
/*      */     
/* 1955 */     value.setIgnoreNotFound(ignoreNotFound);
/* 1956 */     value.setCascadeDeleteEnabled(cascadeOnDelete);
/*      */     
/* 1958 */     if (!optional) {
/* 1959 */       for (Ejb3JoinColumn column : columns) {
/* 1960 */         column.setNullable(false);
/*      */       }
/*      */     }
/* 1963 */     value.setTypeName(inferredData.getClassOrElementName());
/* 1964 */     String propertyName = inferredData.getPropertyName();
/* 1965 */     value.setTypeUsingReflection(propertyHolder.getClassName(), propertyName);
/*      */     
/* 1967 */     ForeignKey fk = (ForeignKey)inferredData.getProperty().getAnnotation(ForeignKey.class);
/* 1968 */     String fkName = fk != null ? fk.name() : "";
/*      */     
/*      */ 
/* 1971 */     if (!BinderHelper.isDefault(fkName)) { value.setForeignKeyName(fkName);
/*      */     }
/* 1973 */     String path = propertyHolder.getPath() + "." + propertyName;
/* 1974 */     FkSecondPass secondPass = new ToOneFkSecondPass(value, columns, (!optional) && (unique), propertyHolder.getEntityOwnerClassName(), path, mappings);
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 1980 */     if (inSecondPass) {
/* 1981 */       secondPass.doSecondPass(mappings.getClasses());
/*      */     }
/*      */     else {
/* 1984 */       mappings.addSecondPass(secondPass);
/*      */     }
/*      */     
/*      */ 
/* 1988 */     Ejb3Column.checkPropertyConsistency(columns, propertyHolder.getEntityName() + propertyName);
/* 1989 */     PropertyBinder binder = new PropertyBinder();
/* 1990 */     binder.setName(propertyName);
/* 1991 */     binder.setValue(value);
/*      */     
/* 1993 */     if (isIdentifierMapper) {
/* 1994 */       binder.setInsertable(false);
/* 1995 */       binder.setUpdatable(false);
/*      */     }
/*      */     else {
/* 1998 */       binder.setInsertable(columns[0].isInsertable());
/* 1999 */       binder.setUpdatable(columns[0].isUpdatable());
/*      */     }
/* 2001 */     binder.setPropertyAccessorName(inferredData.getDefaultAccess());
/* 2002 */     binder.setCascade(cascadeStrategy);
/* 2003 */     Property prop = binder.make();
/*      */     
/* 2005 */     propertyHolder.addProperty(prop, columns);
/*      */   }
/*      */   
/*      */   protected static void defineFetchingStrategy(ToOne toOne, XProperty property) {
/* 2009 */     LazyToOne lazy = (LazyToOne)property.getAnnotation(LazyToOne.class);
/* 2010 */     Fetch fetch = (Fetch)property.getAnnotation(Fetch.class);
/* 2011 */     javax.persistence.ManyToOne manyToOne = (javax.persistence.ManyToOne)property.getAnnotation(javax.persistence.ManyToOne.class);
/* 2012 */     OneToOne oneToOne = (OneToOne)property.getAnnotation(OneToOne.class);
/*      */     FetchType fetchType;
/* 2014 */     if (manyToOne != null) {
/* 2015 */       fetchType = manyToOne.fetch();
/*      */     } else { FetchType fetchType;
/* 2017 */       if (oneToOne != null) {
/* 2018 */         fetchType = oneToOne.fetch();
/*      */       }
/*      */       else {
/* 2021 */         throw new AssertionFailure("Define fetch strategy on a property not annotated with @OneToMany nor @OneToOne");
/*      */       }
/*      */     }
/*      */     FetchType fetchType;
/* 2025 */     if (lazy != null) {
/* 2026 */       toOne.setLazy(lazy.value() != LazyToOneOption.FALSE);
/* 2027 */       toOne.setUnwrapProxy(lazy.value() == LazyToOneOption.NO_PROXY);
/*      */     }
/*      */     else {
/* 2030 */       toOne.setLazy(fetchType == FetchType.LAZY);
/* 2031 */       toOne.setUnwrapProxy(false);
/*      */     }
/* 2033 */     if (fetch != null) {
/* 2034 */       if (fetch.value() == org.hibernate.annotations.FetchMode.JOIN) {
/* 2035 */         toOne.setFetchMode(org.hibernate.FetchMode.JOIN);
/* 2036 */         toOne.setLazy(false);
/* 2037 */         toOne.setUnwrapProxy(false);
/*      */       }
/* 2039 */       else if (fetch.value() == org.hibernate.annotations.FetchMode.SELECT) {
/* 2040 */         toOne.setFetchMode(org.hibernate.FetchMode.SELECT);
/*      */       } else {
/* 2042 */         if (fetch.value() == org.hibernate.annotations.FetchMode.SUBSELECT) {
/* 2043 */           throw new AnnotationException("Use of FetchMode.SUBSELECT not allowed on ToOne associations");
/*      */         }
/*      */         
/* 2046 */         throw new AssertionFailure("Unknown FetchMode: " + fetch.value());
/*      */       }
/*      */     }
/*      */     else {
/* 2050 */       toOne.setFetchMode(getFetchMode(fetchType));
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private static void bindOneToOne(String cascadeStrategy, Ejb3JoinColumn[] joinColumns, boolean optional, org.hibernate.FetchMode fetchMode, boolean ignoreNotFound, boolean cascadeOnDelete, XClass targetEntity, PropertyHolder propertyHolder, PropertyData inferredData, String mappedBy, boolean trueOneToOne, boolean isIdentifierMapper, boolean inSecondPass, ExtendedMappings mappings)
/*      */   {
/* 2068 */     String propertyName = inferredData.getPropertyName();
/* 2069 */     log.debug("Fetching " + propertyName + " with " + fetchMode);
/* 2070 */     boolean mapToPK = true;
/* 2071 */     if (!trueOneToOne)
/*      */     {
/* 2073 */       KeyValue identifier = propertyHolder.getIdentifier();
/* 2074 */       if (identifier == null)
/*      */       {
/*      */ 
/* 2077 */         mapToPK = false;
/*      */       }
/*      */       else {
/* 2080 */         Iterator idColumns = identifier.getColumnIterator();
/* 2081 */         List<String> idColumnNames = new ArrayList();
/*      */         
/* 2083 */         while (idColumns.hasNext()) {
/* 2084 */           org.hibernate.mapping.Column currentColumn = (org.hibernate.mapping.Column)idColumns.next();
/* 2085 */           idColumnNames.add(currentColumn.getName());
/*      */         }
/* 2087 */         for (Ejb3JoinColumn col : joinColumns) {
/* 2088 */           if (!idColumnNames.contains(col.getMappingColumn().getName())) {
/* 2089 */             mapToPK = false;
/* 2090 */             break;
/*      */           }
/*      */         }
/*      */       }
/*      */     }
/* 2095 */     if ((trueOneToOne) || (mapToPK) || (!BinderHelper.isDefault(mappedBy)))
/*      */     {
/*      */ 
/* 2098 */       OneToOneSecondPass secondPass = new OneToOneSecondPass(mappedBy, propertyHolder.getEntityName(), propertyName, propertyHolder, inferredData, targetEntity, ignoreNotFound, cascadeOnDelete, optional, cascadeStrategy, joinColumns, mappings);
/*      */       
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 2105 */       if (inSecondPass) {
/* 2106 */         secondPass.doSecondPass(mappings.getClasses());
/*      */       }
/*      */       else {
/* 2109 */         mappings.addSecondPass(secondPass, BinderHelper.isDefault(mappedBy));
/*      */       }
/*      */       
/*      */ 
/*      */     }
/*      */     else
/*      */     {
/* 2116 */       bindManyToOne(cascadeStrategy, joinColumns, optional, ignoreNotFound, cascadeOnDelete, targetEntity, propertyHolder, inferredData, true, isIdentifierMapper, inSecondPass, mappings);
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private static void bindAny(String cascadeStrategy, Ejb3JoinColumn[] columns, boolean cascadeOnDelete, Nullability nullability, PropertyHolder propertyHolder, PropertyData inferredData, EntityBinder entityBinder, boolean isIdentifierMapper, ExtendedMappings mappings)
/*      */   {
/* 2129 */     org.hibernate.annotations.Any anyAnn = (org.hibernate.annotations.Any)inferredData.getProperty().getAnnotation(org.hibernate.annotations.Any.class);
/* 2130 */     if (anyAnn == null) {
/* 2131 */       throw new AssertionFailure("Missing @Any annotation: " + StringHelper.qualify(propertyHolder.getPath(), inferredData.getPropertyName()));
/*      */     }
/*      */     
/* 2134 */     org.hibernate.mapping.Any value = BinderHelper.buildAnyValue(anyAnn.metaDef(), columns, anyAnn.metaColumn(), inferredData, cascadeOnDelete, nullability, propertyHolder, entityBinder, anyAnn.optional(), mappings);
/*      */     
/*      */ 
/* 2137 */     PropertyBinder binder = new PropertyBinder();
/* 2138 */     binder.setName(inferredData.getPropertyName());
/* 2139 */     binder.setValue(value);
/*      */     
/* 2141 */     binder.setLazy(anyAnn.fetch() == FetchType.LAZY);
/*      */     
/* 2143 */     if (isIdentifierMapper) {
/* 2144 */       binder.setInsertable(false);
/* 2145 */       binder.setUpdatable(false);
/*      */     }
/*      */     else {
/* 2148 */       binder.setInsertable(columns[0].isInsertable());
/* 2149 */       binder.setUpdatable(columns[0].isUpdatable());
/*      */     }
/* 2151 */     binder.setPropertyAccessorName(inferredData.getDefaultAccess());
/* 2152 */     binder.setCascade(cascadeStrategy);
/* 2153 */     Property prop = binder.make();
/*      */     
/* 2155 */     propertyHolder.addProperty(prop, columns);
/*      */   }
/*      */   
/*      */   private static String generatorType(GenerationType generatorEnum) {
/* 2159 */     switch (generatorEnum) {
/*      */     case IDENTITY: 
/* 2161 */       return "identity";
/*      */     case AUTO: 
/* 2163 */       return "native";
/*      */     case TABLE: 
/* 2165 */       return MultipleHiLoPerTableGenerator.class.getName();
/*      */     case SEQUENCE: 
/* 2167 */       return "seqhilo";
/*      */     }
/* 2169 */     throw new AssertionFailure("Unknown GeneratorType: " + generatorEnum);
/*      */   }
/*      */   
/*      */   private static EnumSet<org.hibernate.annotations.CascadeType> convertToHibernateCascadeType(javax.persistence.CascadeType[] ejbCascades) {
/* 2173 */     EnumSet<org.hibernate.annotations.CascadeType> hibernateCascadeSet = EnumSet.noneOf(org.hibernate.annotations.CascadeType.class);
/* 2174 */     if ((ejbCascades != null) && (ejbCascades.length > 0)) {
/* 2175 */       for (javax.persistence.CascadeType cascade : ejbCascades) {
/* 2176 */         switch (cascade) {
/*      */         case ALL: 
/* 2178 */           hibernateCascadeSet.add(org.hibernate.annotations.CascadeType.ALL);
/* 2179 */           break;
/*      */         case PERSIST: 
/* 2181 */           hibernateCascadeSet.add(org.hibernate.annotations.CascadeType.PERSIST);
/* 2182 */           break;
/*      */         case MERGE: 
/* 2184 */           hibernateCascadeSet.add(org.hibernate.annotations.CascadeType.MERGE);
/* 2185 */           break;
/*      */         case REMOVE: 
/* 2187 */           hibernateCascadeSet.add(org.hibernate.annotations.CascadeType.REMOVE);
/* 2188 */           break;
/*      */         case REFRESH: 
/* 2190 */           hibernateCascadeSet.add(org.hibernate.annotations.CascadeType.REFRESH);
/*      */         }
/*      */         
/*      */       }
/*      */     }
/*      */     
/* 2196 */     return hibernateCascadeSet;
/*      */   }
/*      */   
/*      */ 
/*      */   private static String getCascadeStrategy(javax.persistence.CascadeType[] ejbCascades, Cascade hibernateCascadeAnnotation)
/*      */   {
/* 2202 */     EnumSet<org.hibernate.annotations.CascadeType> hibernateCascadeSet = convertToHibernateCascadeType(ejbCascades);
/* 2203 */     org.hibernate.annotations.CascadeType[] hibernateCascades = hibernateCascadeAnnotation == null ? null : hibernateCascadeAnnotation.value();
/*      */     
/*      */ 
/*      */ 
/* 2207 */     if ((hibernateCascades != null) && (hibernateCascades.length > 0)) {
/* 2208 */       for (org.hibernate.annotations.CascadeType cascadeType : hibernateCascades) {
/* 2209 */         hibernateCascadeSet.add(cascadeType);
/*      */       }
/*      */     }
/*      */     
/* 2213 */     StringBuilder cascade = new StringBuilder();
/* 2214 */     Iterator<org.hibernate.annotations.CascadeType> cascadeType = hibernateCascadeSet.iterator();
/* 2215 */     while (cascadeType.hasNext()) {
/* 2216 */       switch ((org.hibernate.annotations.CascadeType)cascadeType.next()) {
/*      */       case ALL: 
/* 2218 */         cascade.append(",").append("all");
/* 2219 */         break;
/*      */       case SAVE_UPDATE: 
/* 2221 */         cascade.append(",").append("save-update");
/* 2222 */         break;
/*      */       case PERSIST: 
/* 2224 */         cascade.append(",").append("persist");
/* 2225 */         break;
/*      */       case MERGE: 
/* 2227 */         cascade.append(",").append("merge");
/* 2228 */         break;
/*      */       case LOCK: 
/* 2230 */         cascade.append(",").append("lock");
/* 2231 */         break;
/*      */       case REFRESH: 
/* 2233 */         cascade.append(",").append("refresh");
/* 2234 */         break;
/*      */       case REPLICATE: 
/* 2236 */         cascade.append(",").append("replicate");
/* 2237 */         break;
/*      */       case EVICT: 
/* 2239 */         cascade.append(",").append("evict");
/* 2240 */         break;
/*      */       case DELETE: 
/* 2242 */         cascade.append(",").append("delete");
/* 2243 */         break;
/*      */       case DELETE_ORPHAN: 
/* 2245 */         cascade.append(",").append("delete-orphan");
/* 2246 */         break;
/*      */       case REMOVE: 
/* 2248 */         cascade.append(",").append("delete");
/*      */       }
/*      */       
/*      */     }
/* 2252 */     return cascade.length() > 0 ? cascade.substring(1) : "none";
/*      */   }
/*      */   
/*      */ 
/*      */   public static org.hibernate.FetchMode getFetchMode(FetchType fetch)
/*      */   {
/* 2258 */     if (fetch == FetchType.EAGER) {
/* 2259 */       return org.hibernate.FetchMode.JOIN;
/*      */     }
/*      */     
/* 2262 */     return org.hibernate.FetchMode.SELECT;
/*      */   }
/*      */   
/*      */   private static HashMap<String, IdGenerator> buildLocalGenerators(XAnnotatedElement annElt, Mappings mappings)
/*      */   {
/* 2267 */     HashMap<String, IdGenerator> generators = new HashMap();
/* 2268 */     TableGenerator tabGen = (TableGenerator)annElt.getAnnotation(TableGenerator.class);
/* 2269 */     SequenceGenerator seqGen = (SequenceGenerator)annElt.getAnnotation(SequenceGenerator.class);
/* 2270 */     GenericGenerator genGen = (GenericGenerator)annElt.getAnnotation(GenericGenerator.class);
/* 2271 */     if (tabGen != null) {
/* 2272 */       IdGenerator idGen = buildIdGenerator(tabGen, mappings);
/* 2273 */       generators.put(idGen.getName(), idGen);
/*      */     }
/* 2275 */     if (seqGen != null) {
/* 2276 */       IdGenerator idGen = buildIdGenerator(seqGen, mappings);
/* 2277 */       generators.put(idGen.getName(), idGen);
/*      */     }
/* 2279 */     if (genGen != null) {
/* 2280 */       IdGenerator idGen = buildIdGenerator(genGen, mappings);
/* 2281 */       generators.put(idGen.getName(), idGen);
/*      */     }
/* 2283 */     return generators;
/*      */   }
/*      */   
/*      */   public static boolean isDefault(XClass clazz, ExtendedMappings mappings) {
/* 2287 */     return mappings.getReflectionManager().equals(clazz, Void.TYPE);
/*      */   }
/*      */   
/*      */ 
/*      */   public static Map<XClass, InheritanceState> buildInheritanceStates(List<XClass> orderedClasses, ReflectionManager reflectionManager)
/*      */   {
/* 2293 */     Map<XClass, InheritanceState> inheritanceStatePerClass = new HashMap(orderedClasses.size());
/*      */     
/*      */ 
/* 2296 */     for (XClass clazz : orderedClasses) {
/* 2297 */       InheritanceState superclassState = InheritanceState.getSuperclassInheritanceState(clazz, inheritanceStatePerClass, reflectionManager);
/*      */       
/*      */ 
/*      */ 
/* 2301 */       InheritanceState state = new InheritanceState(clazz);
/* 2302 */       if (superclassState != null)
/*      */       {
/*      */ 
/* 2305 */         superclassState.hasSons = true;
/* 2306 */         InheritanceState superEntityState = InheritanceState.getSuperEntityInheritanceState(clazz, inheritanceStatePerClass, reflectionManager);
/*      */         
/*      */ 
/*      */ 
/* 2310 */         state.hasParents = (superEntityState != null);
/* 2311 */         boolean nonDefault = (state.type != null) && (!InheritanceType.SINGLE_TABLE.equals(state.type));
/* 2312 */         if (superclassState.type != null) {
/* 2313 */           boolean mixingStrategy = (state.type != null) && (!state.type.equals(superclassState.type));
/* 2314 */           if ((nonDefault) && (mixingStrategy)) {
/* 2315 */             log.warn("Mixing inheritance strategy in a entity hierarchy is not allowed, ignoring sub strategy in: " + clazz.getName());
/*      */           }
/*      */           
/*      */ 
/*      */ 
/* 2320 */           state.type = superclassState.type;
/*      */         }
/*      */       }
/* 2323 */       inheritanceStatePerClass.put(clazz, state);
/*      */     }
/* 2325 */     return inheritanceStatePerClass;
/*      */   }
/*      */ }


/* Location:              C:\Users\anandkum\Desktop\source\hibernate-annotations-3.3.1.GA.jar!\org\hibernate\cfg\AnnotationBinder.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */