/*     */ package org.hibernate.cfg.annotations;
/*     */ 
/*     */ import java.util.ArrayList;
/*     */ import java.util.Collection;
/*     */ import java.util.HashMap;
/*     */ import java.util.Iterator;
/*     */ import java.util.List;
/*     */ import java.util.Map;
/*     */ import java.util.Map.Entry;
/*     */ import java.util.Set;
/*     */ import javax.persistence.JoinColumn;
/*     */ import javax.persistence.JoinTable;
/*     */ import javax.persistence.PrimaryKeyJoinColumn;
/*     */ import javax.persistence.SecondaryTable;
/*     */ import javax.persistence.SecondaryTables;
/*     */ import javax.persistence.UniqueConstraint;
/*     */ import org.apache.commons.logging.Log;
/*     */ import org.apache.commons.logging.LogFactory;
/*     */ import org.hibernate.AnnotationException;
/*     */ import org.hibernate.AssertionFailure;
/*     */ import org.hibernate.EntityMode;
/*     */ import org.hibernate.MappingException;
/*     */ import org.hibernate.annotations.AccessType;
/*     */ import org.hibernate.annotations.BatchSize;
/*     */ import org.hibernate.annotations.Cache;
/*     */ import org.hibernate.annotations.CacheConcurrencyStrategy;
/*     */ import org.hibernate.annotations.FetchMode;
/*     */ import org.hibernate.annotations.ForceDiscriminator;
/*     */ import org.hibernate.annotations.ForeignKey;
/*     */ import org.hibernate.annotations.Immutable;
/*     */ import org.hibernate.annotations.Loader;
/*     */ import org.hibernate.annotations.OptimisticLockType;
/*     */ import org.hibernate.annotations.Persister;
/*     */ import org.hibernate.annotations.PolymorphismType;
/*     */ import org.hibernate.annotations.Proxy;
/*     */ import org.hibernate.annotations.ResultCheckStyle;
/*     */ import org.hibernate.annotations.SQLDelete;
/*     */ import org.hibernate.annotations.SQLDeleteAll;
/*     */ import org.hibernate.annotations.SQLInsert;
/*     */ import org.hibernate.annotations.SQLUpdate;
/*     */ import org.hibernate.annotations.Tables;
/*     */ import org.hibernate.annotations.Tuplizer;
/*     */ import org.hibernate.annotations.Tuplizers;
/*     */ import org.hibernate.annotations.Where;
/*     */ import org.hibernate.annotations.common.reflection.ReflectionManager;
/*     */ import org.hibernate.annotations.common.reflection.XAnnotatedElement;
/*     */ import org.hibernate.annotations.common.reflection.XClass;
/*     */ import org.hibernate.cfg.AnnotationBinder;
/*     */ import org.hibernate.cfg.BinderHelper;
/*     */ import org.hibernate.cfg.Ejb3JoinColumn;
/*     */ import org.hibernate.cfg.ExtendedMappings;
/*     */ import org.hibernate.cfg.InheritanceState;
/*     */ import org.hibernate.cfg.NamingStrategy;
/*     */ import org.hibernate.cfg.PropertyHolder;
/*     */ import org.hibernate.engine.ExecuteUpdateResultCheckStyle;
/*     */ import org.hibernate.engine.FilterDefinition;
/*     */ import org.hibernate.mapping.DependantValue;
/*     */ import org.hibernate.mapping.Join;
/*     */ import org.hibernate.mapping.PersistentClass;
/*     */ import org.hibernate.mapping.RootClass;
/*     */ import org.hibernate.mapping.SimpleValue;
/*     */ import org.hibernate.mapping.TableOwner;
/*     */ import org.hibernate.mapping.Value;
/*     */ import org.hibernate.type.Type;
/*     */ import org.hibernate.util.ReflectHelper;
/*     */ import org.hibernate.util.StringHelper;
/*     */ 
/*     */ 
/*     */ 
/*     */ public class EntityBinder
/*     */ {
/*     */   private String name;
/*     */   private XClass annotatedClass;
/*     */   private PersistentClass persistentClass;
/*     */   private ExtendedMappings mappings;
/*  76 */   private static Log log = LogFactory.getLog(EntityBinder.class);
/*  77 */   private String discriminatorValue = "";
/*  78 */   private boolean isPropertyAnnotated = false;
/*     */   private boolean dynamicInsert;
/*     */   private boolean dynamicUpdate;
/*     */   private boolean explicitHibernateEntityAnnotation;
/*     */   private OptimisticLockType optimisticLockType;
/*     */   private PolymorphismType polymorphismType;
/*     */   private boolean selectBeforeUpdate;
/*     */   private int batchSize;
/*     */   private boolean lazy;
/*     */   private XClass proxyClass;
/*     */   private String where;
/*  89 */   private Map<String, Join> secondaryTables = new HashMap();
/*  90 */   private Map<String, Object> secondaryTableJoins = new HashMap();
/*     */   private String cacheConcurrentStrategy;
/*     */   private String cacheRegion;
/*  93 */   private Map<String, String> filters = new HashMap();
/*     */   private InheritanceState inheritanceState;
/*     */   private boolean ignoreIdAnnotations;
/*     */   private boolean cacheLazyProperty;
/*     */   private String propertyAccessor;
/*     */   
/*     */   public boolean isPropertyAnnotated() {
/* 100 */     return this.isPropertyAnnotated;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public EntityBinder() {}
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   public EntityBinder(javax.persistence.Entity ejb3Ann, org.hibernate.annotations.Entity hibAnn, XClass annotatedClass, PersistentClass persistentClass, ExtendedMappings mappings)
/*     */   {
/* 114 */     this.mappings = mappings;
/* 115 */     this.persistentClass = persistentClass;
/* 116 */     this.annotatedClass = annotatedClass;
/* 117 */     bindEjb3Annotation(ejb3Ann);
/* 118 */     bindHibernateAnnotation(hibAnn);
/*     */   }
/*     */   
/*     */   private void bindHibernateAnnotation(org.hibernate.annotations.Entity hibAnn) {
/* 122 */     if (hibAnn != null) {
/* 123 */       this.dynamicInsert = hibAnn.dynamicInsert();
/* 124 */       this.dynamicUpdate = hibAnn.dynamicUpdate();
/* 125 */       this.optimisticLockType = hibAnn.optimisticLock();
/* 126 */       this.selectBeforeUpdate = hibAnn.selectBeforeUpdate();
/* 127 */       this.polymorphismType = hibAnn.polymorphism();
/* 128 */       this.explicitHibernateEntityAnnotation = true;
/*     */ 
/*     */     }
/*     */     else
/*     */     {
/* 133 */       this.dynamicInsert = false;
/* 134 */       this.dynamicUpdate = false;
/* 135 */       this.optimisticLockType = OptimisticLockType.VERSION;
/* 136 */       this.polymorphismType = PolymorphismType.IMPLICIT;
/* 137 */       this.selectBeforeUpdate = false;
/*     */     }
/*     */   }
/*     */   
/*     */   private void bindEjb3Annotation(javax.persistence.Entity ejb3Ann) {
/* 142 */     if (ejb3Ann == null) throw new AssertionFailure("@Entity should always be not null");
/* 143 */     if (BinderHelper.isDefault(ejb3Ann.name())) {
/* 144 */       this.name = StringHelper.unqualify(this.annotatedClass.getName());
/*     */     }
/*     */     else {
/* 147 */       this.name = ejb3Ann.name();
/*     */     }
/*     */   }
/*     */   
/*     */   public void setDiscriminatorValue(String discriminatorValue) {
/* 152 */     this.discriminatorValue = discriminatorValue;
/*     */   }
/*     */   
/*     */   public void bindEntity() {
/* 156 */     this.persistentClass.setAbstract(Boolean.valueOf(this.annotatedClass.isAbstract()));
/* 157 */     this.persistentClass.setClassName(this.annotatedClass.getName());
/* 158 */     this.persistentClass.setNodeName(this.name);
/*     */     
/* 160 */     this.persistentClass.setEntityName(this.annotatedClass.getName());
/* 161 */     bindDiscriminatorValue();
/*     */     
/* 163 */     this.persistentClass.setLazy(this.lazy);
/* 164 */     if (this.proxyClass != null) {
/* 165 */       this.persistentClass.setProxyInterfaceName(this.proxyClass.getName());
/*     */     }
/* 167 */     this.persistentClass.setDynamicInsert(this.dynamicInsert);
/* 168 */     this.persistentClass.setDynamicUpdate(this.dynamicUpdate);
/*     */     
/* 170 */     if ((this.persistentClass instanceof RootClass)) {
/* 171 */       RootClass rootClass = (RootClass)this.persistentClass;
/* 172 */       boolean mutable = true;
/*     */       
/* 174 */       if (this.annotatedClass.isAnnotationPresent(Immutable.class)) {
/* 175 */         mutable = false;
/*     */       }
/*     */       else {
/* 178 */         org.hibernate.annotations.Entity entityAnn = (org.hibernate.annotations.Entity)this.annotatedClass.getAnnotation(org.hibernate.annotations.Entity.class);
/*     */         
/* 180 */         if (entityAnn != null) {
/* 181 */           mutable = entityAnn.mutable();
/*     */         }
/*     */       }
/* 184 */       rootClass.setMutable(mutable);
/* 185 */       rootClass.setExplicitPolymorphism(isExplicitPolymorphism(this.polymorphismType));
/* 186 */       if (StringHelper.isNotEmpty(this.where)) rootClass.setWhere(this.where);
/* 187 */       if (this.cacheConcurrentStrategy != null) {
/* 188 */         rootClass.setCacheConcurrencyStrategy(this.cacheConcurrentStrategy);
/* 189 */         rootClass.setCacheRegionName(this.cacheRegion);
/* 190 */         rootClass.setLazyPropertiesCacheable(this.cacheLazyProperty);
/*     */       }
/* 192 */       rootClass.setForceDiscriminator(this.annotatedClass.isAnnotationPresent(ForceDiscriminator.class));
/*     */     }
/*     */     else {
/* 195 */       if (this.explicitHibernateEntityAnnotation) {
/* 196 */         log.warn("@org.hibernate.annotations.Entity used on a non root entity: ignored for " + this.annotatedClass.getName());
/*     */       }
/*     */       
/* 199 */       if (this.annotatedClass.isAnnotationPresent(Immutable.class)) {
/* 200 */         log.warn("@Immutable used on a non root entity: ignored for " + this.annotatedClass.getName());
/*     */       }
/*     */     }
/*     */     
/* 204 */     this.persistentClass.setOptimisticLockMode(getVersioning(this.optimisticLockType));
/* 205 */     this.persistentClass.setSelectBeforeUpdate(this.selectBeforeUpdate);
/*     */     
/*     */ 
/*     */ 
/* 209 */     Persister persisterAnn = (Persister)this.annotatedClass.getAnnotation(Persister.class);
/* 210 */     Class persister = null;
/* 211 */     if (persisterAnn != null) {
/* 212 */       persister = persisterAnn.impl();
/*     */     }
/*     */     else {
/* 215 */       org.hibernate.annotations.Entity entityAnn = (org.hibernate.annotations.Entity)this.annotatedClass.getAnnotation(org.hibernate.annotations.Entity.class);
/* 216 */       if ((entityAnn != null) && (!BinderHelper.isDefault(entityAnn.persister()))) {
/*     */         try {
/* 218 */           persister = ReflectHelper.classForName(entityAnn.persister());
/*     */         }
/*     */         catch (ClassNotFoundException cnfe) {
/* 221 */           throw new AnnotationException("Could not find persister class: " + persister);
/*     */         }
/*     */       }
/*     */     }
/* 225 */     if (persister != null) { this.persistentClass.setEntityPersisterClass(persister);
/*     */     }
/* 227 */     this.persistentClass.setBatchSize(this.batchSize);
/*     */     
/*     */ 
/* 230 */     SQLInsert sqlInsert = (SQLInsert)this.annotatedClass.getAnnotation(SQLInsert.class);
/* 231 */     SQLUpdate sqlUpdate = (SQLUpdate)this.annotatedClass.getAnnotation(SQLUpdate.class);
/* 232 */     SQLDelete sqlDelete = (SQLDelete)this.annotatedClass.getAnnotation(SQLDelete.class);
/* 233 */     SQLDeleteAll sqlDeleteAll = (SQLDeleteAll)this.annotatedClass.getAnnotation(SQLDeleteAll.class);
/* 234 */     Loader loader = (Loader)this.annotatedClass.getAnnotation(Loader.class);
/* 235 */     if (sqlInsert != null) {
/* 236 */       this.persistentClass.setCustomSQLInsert(sqlInsert.sql().trim(), sqlInsert.callable(), ExecuteUpdateResultCheckStyle.parse(sqlInsert.check().toString().toLowerCase()));
/*     */     }
/*     */     
/*     */ 
/*     */ 
/* 241 */     if (sqlUpdate != null) {
/* 242 */       this.persistentClass.setCustomSQLUpdate(sqlUpdate.sql(), sqlUpdate.callable(), ExecuteUpdateResultCheckStyle.parse(sqlUpdate.check().toString().toLowerCase()));
/*     */     }
/*     */     
/*     */ 
/* 246 */     if (sqlDelete != null) {
/* 247 */       this.persistentClass.setCustomSQLDelete(sqlDelete.sql(), sqlDelete.callable(), ExecuteUpdateResultCheckStyle.parse(sqlDelete.check().toString().toLowerCase()));
/*     */     }
/*     */     
/*     */ 
/* 251 */     if (sqlDeleteAll != null) {
/* 252 */       this.persistentClass.setCustomSQLDelete(sqlDeleteAll.sql(), sqlDeleteAll.callable(), ExecuteUpdateResultCheckStyle.parse(sqlDeleteAll.check().toString().toLowerCase()));
/*     */     }
/*     */     
/*     */ 
/* 256 */     if (loader != null) {
/* 257 */       this.persistentClass.setLoaderName(loader.namedQuery());
/*     */     }
/*     */     
/*     */ 
/* 261 */     if (this.annotatedClass.isAnnotationPresent(Tuplizers.class)) {
/* 262 */       for (Tuplizer tuplizer : ((Tuplizers)this.annotatedClass.getAnnotation(Tuplizers.class)).value()) {
/* 263 */         EntityMode mode = EntityMode.parse(tuplizer.entityMode());
/* 264 */         this.persistentClass.addTuplizer(mode, tuplizer.impl().getName());
/*     */       }
/*     */     }
/* 267 */     if (this.annotatedClass.isAnnotationPresent(Tuplizer.class)) {
/* 268 */       Tuplizer tuplizer = (Tuplizer)this.annotatedClass.getAnnotation(Tuplizer.class);
/* 269 */       EntityMode mode = EntityMode.parse(tuplizer.entityMode());
/* 270 */       this.persistentClass.addTuplizer(mode, tuplizer.impl().getName());
/*     */     }
/*     */     
/* 273 */     if (!this.inheritanceState.hasParents) {
/* 274 */       Iterator<Map.Entry<String, String>> iter = this.filters.entrySet().iterator();
/* 275 */       while (iter.hasNext()) {
/* 276 */         Map.Entry<String, String> filter = (Map.Entry)iter.next();
/* 277 */         String filterName = (String)filter.getKey();
/* 278 */         String cond = (String)filter.getValue();
/* 279 */         if (BinderHelper.isDefault(cond)) {
/* 280 */           FilterDefinition definition = this.mappings.getFilterDefinition(filterName);
/* 281 */           cond = definition == null ? null : definition.getDefaultFilterCondition();
/* 282 */           if (StringHelper.isEmpty(cond)) {
/* 283 */             throw new AnnotationException("no filter condition found for filter " + filterName + " in " + this.name);
/*     */           }
/*     */         }
/*     */         
/*     */ 
/* 288 */         this.persistentClass.addFilter(filterName, cond);
/*     */       }
/*     */       
/*     */     }
/* 292 */     else if (this.filters.size() > 0) {
/* 293 */       log.warn("@Filter not allowed on subclasses (ignored): " + this.persistentClass.getEntityName());
/*     */     }
/*     */     
/* 296 */     log.debug("Import with entity name=" + this.name);
/*     */     try {
/* 298 */       this.mappings.addImport(this.persistentClass.getEntityName(), this.name);
/* 299 */       String entityName = this.persistentClass.getEntityName();
/* 300 */       if (!entityName.equals(this.name)) {
/* 301 */         this.mappings.addImport(entityName, entityName);
/*     */       }
/*     */     }
/*     */     catch (MappingException me) {
/* 305 */       throw new AnnotationException("Use of the same entity name twice: " + this.name, me);
/*     */     }
/*     */   }
/*     */   
/*     */   public void bindDiscriminatorValue() {
/* 310 */     if (StringHelper.isEmpty(this.discriminatorValue)) {
/* 311 */       Value discriminator = this.persistentClass.getDiscriminator();
/* 312 */       if (discriminator == null) {
/* 313 */         this.persistentClass.setDiscriminatorValue(this.name);
/*     */       } else {
/* 315 */         if ("character".equals(discriminator.getType().getName())) {
/* 316 */           throw new AnnotationException("Using default @DiscriminatorValue for a discriminator of type CHAR is not safe");
/*     */         }
/*     */         
/*     */ 
/* 320 */         if ("integer".equals(discriminator.getType().getName())) {
/* 321 */           this.persistentClass.setDiscriminatorValue(String.valueOf(this.name.hashCode()));
/*     */         }
/*     */         else {
/* 324 */           this.persistentClass.setDiscriminatorValue(this.name);
/*     */         }
/*     */       }
/*     */     }
/*     */     else {
/* 329 */       this.persistentClass.setDiscriminatorValue(this.discriminatorValue);
/*     */     }
/*     */   }
/*     */   
/*     */   int getVersioning(OptimisticLockType type) {
/* 334 */     switch (type) {
/*     */     case VERSION: 
/* 336 */       return 0;
/*     */     case NONE: 
/* 338 */       return -1;
/*     */     case DIRTY: 
/* 340 */       return 1;
/*     */     case ALL: 
/* 342 */       return 2;
/*     */     }
/* 344 */     throw new AssertionFailure("optimistic locking not supported: " + type);
/*     */   }
/*     */   
/*     */   private boolean isExplicitPolymorphism(PolymorphismType type)
/*     */   {
/* 349 */     switch (type) {
/*     */     case IMPLICIT: 
/* 351 */       return false;
/*     */     case EXPLICIT: 
/* 353 */       return true;
/*     */     }
/* 355 */     throw new AssertionFailure("Unknown polymorphism type: " + type);
/*     */   }
/*     */   
/*     */   public void setBatchSize(BatchSize sizeAnn)
/*     */   {
/* 360 */     if (sizeAnn != null) {
/* 361 */       this.batchSize = sizeAnn.size();
/*     */     }
/*     */     else {
/* 364 */       this.batchSize = -1;
/*     */     }
/*     */   }
/*     */   
/*     */   public void setProxy(Proxy proxy) {
/* 369 */     if (proxy != null) {
/* 370 */       this.lazy = proxy.lazy();
/* 371 */       if (!this.lazy) {
/* 372 */         this.proxyClass = null;
/*     */ 
/*     */       }
/* 375 */       else if (AnnotationBinder.isDefault(this.mappings.getReflectionManager().toXClass(proxy.proxyClass()), this.mappings))
/*     */       {
/*     */ 
/* 378 */         this.proxyClass = this.annotatedClass;
/*     */       }
/*     */       else {
/* 381 */         this.proxyClass = this.mappings.getReflectionManager().toXClass(proxy.proxyClass());
/*     */       }
/*     */     }
/*     */     else
/*     */     {
/* 386 */       this.lazy = true;
/* 387 */       this.proxyClass = this.annotatedClass;
/*     */     }
/*     */   }
/*     */   
/*     */   public void setWhere(Where whereAnn) {
/* 392 */     if (whereAnn != null) {
/* 393 */       this.where = whereAnn.clause();
/*     */     }
/*     */   }
/*     */   
/*     */   private String getClassTableName(String tableName) {
/* 398 */     if (StringHelper.isEmpty(tableName)) {
/* 399 */       return this.mappings.getNamingStrategy().classToTableName(this.name);
/*     */     }
/*     */     
/* 402 */     return this.mappings.getNamingStrategy().tableName(tableName);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public void bindTable(String schema, String catalog, String tableName, List uniqueConstraints, String constraints, org.hibernate.mapping.Table denormalizedSuperclassTable)
/*     */   {
/* 411 */     String logicalName = StringHelper.isNotEmpty(tableName) ? tableName : StringHelper.unqualify(this.name);
/*     */     
/*     */ 
/* 414 */     org.hibernate.mapping.Table table = TableBinder.fillTable(schema, catalog, getClassTableName(tableName), logicalName, this.persistentClass.isAbstract().booleanValue(), uniqueConstraints, constraints, denormalizedSuperclassTable, this.mappings);
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 422 */     if ((this.persistentClass instanceof TableOwner)) {
/* 423 */       if (log.isInfoEnabled()) {
/* 424 */         log.info("Bind entity " + this.persistentClass.getEntityName() + " on table " + table.getName());
/*     */       }
/* 426 */       ((TableOwner)this.persistentClass).setTable(table);
/*     */     }
/*     */     else {
/* 429 */       throw new AssertionFailure("binding a table for a subclass");
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   public void finalSecondaryTableBinding(PropertyHolder propertyHolder)
/*     */   {
/* 438 */     Iterator joins = this.secondaryTables.values().iterator();
/* 439 */     Iterator joinColumns = this.secondaryTableJoins.values().iterator();
/*     */     
/* 441 */     while (joins.hasNext()) {
/* 442 */       Object uncastedColumn = joinColumns.next();
/* 443 */       Join join = (Join)joins.next();
/* 444 */       createPrimaryColumnsToSecondaryTable(uncastedColumn, propertyHolder, join);
/*     */     }
/* 446 */     this.mappings.addJoins(this.persistentClass, this.secondaryTables);
/*     */   }
/*     */   
/*     */   private void createPrimaryColumnsToSecondaryTable(Object uncastedColumn, PropertyHolder propertyHolder, Join join)
/*     */   {
/* 451 */     PrimaryKeyJoinColumn[] pkColumnsAnn = null;
/* 452 */     JoinColumn[] joinColumnsAnn = null;
/* 453 */     if ((uncastedColumn instanceof PrimaryKeyJoinColumn[])) {
/* 454 */       pkColumnsAnn = (PrimaryKeyJoinColumn[])uncastedColumn;
/*     */     }
/* 456 */     if ((uncastedColumn instanceof JoinColumn[]))
/* 457 */       joinColumnsAnn = (JoinColumn[])uncastedColumn;
/*     */     Ejb3JoinColumn[] ejb3JoinColumns;
/* 459 */     if ((pkColumnsAnn == null) && (joinColumnsAnn == null)) {
/* 460 */       Ejb3JoinColumn[] ejb3JoinColumns = new Ejb3JoinColumn[1];
/* 461 */       ejb3JoinColumns[0] = Ejb3JoinColumn.buildJoinColumn(null, null, this.persistentClass.getIdentifier(), this.secondaryTables, propertyHolder, this.mappings);
/*     */ 
/*     */ 
/*     */ 
/*     */     }
/*     */     else
/*     */     {
/*     */ 
/*     */ 
/* 470 */       int nbrOfJoinColumns = pkColumnsAnn != null ? pkColumnsAnn.length : joinColumnsAnn.length;
/*     */       
/*     */ 
/* 473 */       if (nbrOfJoinColumns == 0) {
/* 474 */         Ejb3JoinColumn[] ejb3JoinColumns = new Ejb3JoinColumn[1];
/* 475 */         ejb3JoinColumns[0] = Ejb3JoinColumn.buildJoinColumn(null, null, this.persistentClass.getIdentifier(), this.secondaryTables, propertyHolder, this.mappings);
/*     */ 
/*     */ 
/*     */ 
/*     */       }
/*     */       else
/*     */       {
/*     */ 
/*     */ 
/* 484 */         ejb3JoinColumns = new Ejb3JoinColumn[nbrOfJoinColumns];
/* 485 */         if (pkColumnsAnn != null) {
/* 486 */           for (int colIndex = 0; colIndex < nbrOfJoinColumns; colIndex++) {
/* 487 */             ejb3JoinColumns[colIndex] = Ejb3JoinColumn.buildJoinColumn(pkColumnsAnn[colIndex], null, this.persistentClass.getIdentifier(), this.secondaryTables, propertyHolder, this.mappings);
/*     */ 
/*     */ 
/*     */           }
/*     */           
/*     */ 
/*     */         }
/*     */         else
/*     */         {
/*     */ 
/* 497 */           for (int colIndex = 0; colIndex < nbrOfJoinColumns; colIndex++) {
/* 498 */             ejb3JoinColumns[colIndex] = Ejb3JoinColumn.buildJoinColumn(null, joinColumnsAnn[colIndex], this.persistentClass.getIdentifier(), this.secondaryTables, propertyHolder, this.mappings);
/*     */           }
/*     */         }
/*     */       }
/*     */     }
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 510 */     for (Ejb3JoinColumn joinColumn : ejb3JoinColumns) {
/* 511 */       joinColumn.forceNotNull();
/*     */     }
/* 513 */     bindJoinToPersistentClass(join, ejb3JoinColumns);
/*     */   }
/*     */   
/*     */ 
/*     */   private void bindJoinToPersistentClass(Join join, Ejb3JoinColumn[] ejb3JoinColumns)
/*     */   {
/* 519 */     SimpleValue key = new DependantValue(join.getTable(), this.persistentClass.getIdentifier());
/* 520 */     join.setKey(key);
/* 521 */     setFKNameIfDefined(join);
/* 522 */     key.setCascadeDeleteEnabled(false);
/* 523 */     TableBinder.bindFk(this.persistentClass, null, ejb3JoinColumns, key, false, this.mappings);
/* 524 */     join.createPrimaryKey();
/* 525 */     join.createForeignKey();
/* 526 */     this.persistentClass.addJoin(join);
/*     */   }
/*     */   
/*     */   private void setFKNameIfDefined(Join join) {
/* 530 */     org.hibernate.annotations.Table matchingTable = findMatchingComplimentTableAnnotation(join);
/* 531 */     if ((matchingTable != null) && (!BinderHelper.isDefault(matchingTable.foreignKey().name()))) {
/* 532 */       ((SimpleValue)join.getKey()).setForeignKeyName(matchingTable.foreignKey().name());
/*     */     }
/*     */   }
/*     */   
/*     */   private org.hibernate.annotations.Table findMatchingComplimentTableAnnotation(Join join) {
/* 537 */     String tableName = join.getTable().getQuotedName();
/* 538 */     org.hibernate.annotations.Table table = (org.hibernate.annotations.Table)this.annotatedClass.getAnnotation(org.hibernate.annotations.Table.class);
/* 539 */     org.hibernate.annotations.Table matchingTable = null;
/* 540 */     if ((table != null) && (tableName.equals(table.appliesTo()))) {
/* 541 */       matchingTable = table;
/*     */     }
/*     */     else {
/* 544 */       Tables tables = (Tables)this.annotatedClass.getAnnotation(Tables.class);
/* 545 */       if (tables != null) {
/* 546 */         for (org.hibernate.annotations.Table current : tables.value()) {
/* 547 */           if (tableName.equals(current.appliesTo())) {
/* 548 */             matchingTable = current;
/* 549 */             break;
/*     */           }
/*     */         }
/*     */       }
/*     */     }
/* 554 */     return matchingTable;
/*     */   }
/*     */   
/*     */ 
/*     */   public void firstLevelSecondaryTablesBinding(SecondaryTable secTable, SecondaryTables secTables)
/*     */   {
/* 560 */     if (secTables != null)
/*     */     {
/* 562 */       for (SecondaryTable tab : secTables.value()) {
/* 563 */         addJoin(tab, null, null, false);
/*     */       }
/*     */       
/*     */     }
/* 567 */     else if (secTable != null) { addJoin(secTable, null, null, false);
/*     */     }
/*     */   }
/*     */   
/*     */   public Join addJoin(JoinTable joinTable, PropertyHolder holder, boolean noDelayInPkColumnCreation)
/*     */   {
/* 573 */     return addJoin(null, joinTable, holder, noDelayInPkColumnCreation);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   private Join addJoin(SecondaryTable secondaryTable, JoinTable joinTable, PropertyHolder propertyHolder, boolean noDelayInPkColumnCreation)
/*     */   {
/* 583 */     Join join = new Join();
/* 584 */     join.setPersistentClass(this.persistentClass);
/*     */     
/*     */ 
/*     */     UniqueConstraint[] uniqueConstraintsAnn;
/*     */     
/*     */ 
/* 590 */     if (secondaryTable != null) {
/* 591 */       String schema = secondaryTable.schema();
/* 592 */       String catalog = secondaryTable.catalog();
/* 593 */       String table = secondaryTable.name();
/* 594 */       String realTable = this.mappings.getNamingStrategy().tableName(table);
/* 595 */       uniqueConstraintsAnn = secondaryTable.uniqueConstraints();
/*     */     } else { UniqueConstraint[] uniqueConstraintsAnn;
/* 597 */       if (joinTable != null) {
/* 598 */         String schema = joinTable.schema();
/* 599 */         String catalog = joinTable.catalog();
/* 600 */         String table = joinTable.name();
/* 601 */         String realTable = this.mappings.getNamingStrategy().tableName(table);
/* 602 */         uniqueConstraintsAnn = joinTable.uniqueConstraints();
/*     */       }
/*     */       else {
/* 605 */         throw new AssertionFailure("Both JoinTable and SecondaryTable are null"); } }
/*     */     UniqueConstraint[] uniqueConstraintsAnn;
/* 607 */     String realTable; String table; String catalog; String schema; List uniqueConstraints = new ArrayList(uniqueConstraintsAnn == null ? 0 : uniqueConstraintsAnn.length);
/*     */     
/*     */ 
/* 610 */     if ((uniqueConstraintsAnn != null) && (uniqueConstraintsAnn.length != 0)) {
/* 611 */       for (UniqueConstraint uc : uniqueConstraintsAnn) {
/* 612 */         uniqueConstraints.add(uc.columnNames());
/*     */       }
/*     */     }
/* 615 */     org.hibernate.mapping.Table tableMapping = TableBinder.fillTable(schema, catalog, realTable, table, false, uniqueConstraints, null, null, this.mappings);
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 622 */     join.setTable(tableMapping);
/*     */     
/*     */ 
/*     */ 
/* 626 */     Object joinColumns = null;
/*     */     
/* 628 */     if (secondaryTable != null) {
/* 629 */       joinColumns = secondaryTable.pkJoinColumns();
/*     */     }
/* 631 */     else if (joinTable != null) {
/* 632 */       joinColumns = joinTable.joinColumns();
/*     */     }
/* 634 */     if (log.isInfoEnabled()) {
/* 635 */       log.info("Adding secondary table to entity " + this.persistentClass.getEntityName() + " -> " + join.getTable().getName());
/*     */     }
/*     */     
/*     */ 
/*     */ 
/*     */ 
/* 641 */     org.hibernate.annotations.Table matchingTable = findMatchingComplimentTableAnnotation(join);
/* 642 */     if (matchingTable != null) {
/* 643 */       join.setSequentialSelect(FetchMode.JOIN != matchingTable.fetch());
/* 644 */       join.setInverse(matchingTable.inverse());
/* 645 */       join.setOptional(matchingTable.optional());
/* 646 */       if (!BinderHelper.isDefault(matchingTable.sqlInsert().sql())) {
/* 647 */         join.setCustomSQLInsert(matchingTable.sqlInsert().sql().trim(), matchingTable.sqlInsert().callable(), ExecuteUpdateResultCheckStyle.parse(matchingTable.sqlInsert().check().toString().toLowerCase()));
/*     */       }
/*     */       
/*     */ 
/*     */ 
/* 652 */       if (!BinderHelper.isDefault(matchingTable.sqlUpdate().sql())) {
/* 653 */         join.setCustomSQLUpdate(matchingTable.sqlUpdate().sql().trim(), matchingTable.sqlUpdate().callable(), ExecuteUpdateResultCheckStyle.parse(matchingTable.sqlUpdate().check().toString().toLowerCase()));
/*     */       }
/*     */       
/*     */ 
/*     */ 
/* 658 */       if (!BinderHelper.isDefault(matchingTable.sqlDelete().sql())) {
/* 659 */         join.setCustomSQLDelete(matchingTable.sqlDelete().sql().trim(), matchingTable.sqlDelete().callable(), ExecuteUpdateResultCheckStyle.parse(matchingTable.sqlDelete().check().toString().toLowerCase()));
/*     */       }
/*     */       
/*     */ 
/*     */     }
/*     */     else
/*     */     {
/*     */ 
/* 667 */       join.setSequentialSelect(false);
/* 668 */       join.setInverse(false);
/* 669 */       join.setOptional(true);
/*     */     }
/*     */     
/* 672 */     if (noDelayInPkColumnCreation) {
/* 673 */       createPrimaryColumnsToSecondaryTable(joinColumns, propertyHolder, join);
/*     */     }
/*     */     else {
/* 676 */       this.secondaryTables.put(table, join);
/* 677 */       this.secondaryTableJoins.put(table, joinColumns);
/*     */     }
/* 679 */     return join;
/*     */   }
/*     */   
/*     */   public Map<String, Join> getSecondaryTables() {
/* 683 */     return this.secondaryTables;
/*     */   }
/*     */   
/*     */   public void setCache(Cache cacheAnn) {
/* 687 */     if (cacheAnn != null) {
/* 688 */       this.cacheRegion = (BinderHelper.isDefault(cacheAnn.region()) ? null : cacheAnn.region());
/*     */       
/*     */ 
/* 691 */       this.cacheConcurrentStrategy = getCacheConcurrencyStrategy(cacheAnn.usage());
/* 692 */       if ("all".equalsIgnoreCase(cacheAnn.include())) {
/* 693 */         this.cacheLazyProperty = true;
/*     */       }
/* 695 */       else if ("non-lazy".equalsIgnoreCase(cacheAnn.include())) {
/* 696 */         this.cacheLazyProperty = false;
/*     */       }
/*     */       else {
/* 699 */         throw new AnnotationException("Unknown lazy property annotations: " + cacheAnn.include());
/*     */       }
/*     */     }
/*     */     else {
/* 703 */       this.cacheConcurrentStrategy = null;
/* 704 */       this.cacheRegion = null;
/* 705 */       this.cacheLazyProperty = true;
/*     */     }
/*     */   }
/*     */   
/*     */   public static String getCacheConcurrencyStrategy(CacheConcurrencyStrategy strategy) {
/* 710 */     switch (strategy) {
/*     */     case NONE: 
/* 712 */       return null;
/*     */     case READ_ONLY: 
/* 714 */       return "read-only";
/*     */     case READ_WRITE: 
/* 716 */       return "read-write";
/*     */     case NONSTRICT_READ_WRITE: 
/* 718 */       return "nonstrict-read-write";
/*     */     case TRANSACTIONAL: 
/* 720 */       return "transactional";
/*     */     }
/* 722 */     throw new AssertionFailure("CacheConcurrencyStrategy unknown: " + strategy);
/*     */   }
/*     */   
/*     */   public void addFilter(String name, String condition)
/*     */   {
/* 727 */     this.filters.put(name, condition);
/*     */   }
/*     */   
/*     */   public void setInheritanceState(InheritanceState inheritanceState) {
/* 731 */     this.inheritanceState = inheritanceState;
/*     */   }
/*     */   
/*     */   public boolean isIgnoreIdAnnotations() {
/* 735 */     return this.ignoreIdAnnotations;
/*     */   }
/*     */   
/*     */   public void setIgnoreIdAnnotations(boolean ignoreIdAnnotations) {
/* 739 */     this.ignoreIdAnnotations = ignoreIdAnnotations;
/*     */   }
/*     */   
/*     */   public void processComplementaryTableDefinitions(org.hibernate.annotations.Table table)
/*     */   {
/* 744 */     if (table == null) return;
/* 745 */     String appliedTable = table.appliesTo();
/* 746 */     Iterator tables = this.persistentClass.getTableClosureIterator();
/* 747 */     org.hibernate.mapping.Table hibTable = null;
/* 748 */     while (tables.hasNext()) {
/* 749 */       org.hibernate.mapping.Table pcTable = (org.hibernate.mapping.Table)tables.next();
/* 750 */       if (pcTable.getQuotedName().equals(appliedTable))
/*     */       {
/* 752 */         hibTable = pcTable;
/* 753 */         break;
/*     */       }
/* 755 */       hibTable = null;
/*     */     }
/* 757 */     if (hibTable == null)
/*     */     {
/* 759 */       for (Join join : this.secondaryTables.values()) {
/* 760 */         if (join.getTable().getQuotedName().equals(appliedTable)) {
/* 761 */           hibTable = join.getTable();
/* 762 */           break;
/*     */         }
/*     */       }
/*     */     }
/* 766 */     if (hibTable == null) {
/* 767 */       throw new AnnotationException("@org.hibernate.annotations.Table references an unknown table: " + appliedTable);
/*     */     }
/*     */     
/*     */ 
/* 771 */     if (!BinderHelper.isDefault(table.comment())) hibTable.setComment(table.comment());
/* 772 */     TableBinder.addIndexes(hibTable, table.indexes(), this.mappings);
/*     */   }
/*     */   
/*     */   public void processComplementaryTableDefinitions(Tables tables) {
/* 776 */     if (tables == null) return;
/* 777 */     for (org.hibernate.annotations.Table table : tables.value()) {
/* 778 */       processComplementaryTableDefinitions(table);
/*     */     }
/*     */   }
/*     */   
/*     */   public void setPropertyAnnotated(boolean propertyAnnotated) {
/* 783 */     this.isPropertyAnnotated = propertyAnnotated;
/*     */   }
/*     */   
/*     */   public String getPropertyAccessor() {
/* 787 */     return this.propertyAccessor;
/*     */   }
/*     */   
/*     */   public void setPropertyAccessor(String propertyAccessor) {
/* 791 */     this.propertyAccessor = propertyAccessor;
/*     */   }
/*     */   
/*     */   public boolean isPropertyAnnotated(XAnnotatedElement element) {
/* 795 */     AccessType access = (AccessType)element.getAnnotation(AccessType.class);
/* 796 */     if (access == null) return this.isPropertyAnnotated;
/* 797 */     String propertyAccessor = access.value();
/* 798 */     if ("property".equals(propertyAccessor)) {
/* 799 */       return Boolean.TRUE.booleanValue();
/*     */     }
/* 801 */     if ("field".equals(propertyAccessor)) {
/* 802 */       return Boolean.FALSE.booleanValue();
/*     */     }
/*     */     
/* 805 */     return this.isPropertyAnnotated;
/*     */   }
/*     */   
/*     */   public String getPropertyAccessor(XAnnotatedElement element)
/*     */   {
/* 810 */     AccessType access = (AccessType)element.getAnnotation(AccessType.class);
/* 811 */     if (access == null) return this.propertyAccessor;
/* 812 */     return access.value();
/*     */   }
/*     */ }


/* Location:              C:\Users\anandkum\Desktop\source\hibernate-annotations-3.3.1.GA.jar!\org\hibernate\cfg\annotations\EntityBinder.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */