/*     */ package org.hibernate.cfg.annotations;
/*     */ 
/*     */ import java.util.HashMap;
/*     */ import javax.persistence.QueryHint;
/*     */ import javax.persistence.SqlResultSetMapping;
/*     */ import javax.persistence.SqlResultSetMappings;
/*     */ import org.apache.commons.logging.Log;
/*     */ import org.apache.commons.logging.LogFactory;
/*     */ import org.hibernate.AnnotationException;
/*     */ import org.hibernate.AssertionFailure;
/*     */ import org.hibernate.CacheMode;
/*     */ import org.hibernate.FlushMode;
/*     */ import org.hibernate.LockMode;
/*     */ import org.hibernate.annotations.CacheModeType;
/*     */ import org.hibernate.annotations.FlushModeType;
/*     */ import org.hibernate.cfg.BinderHelper;
/*     */ import org.hibernate.cfg.ExtendedMappings;
/*     */ import org.hibernate.cfg.NotYetImplementedException;
/*     */ import org.hibernate.engine.NamedQueryDefinition;
/*     */ import org.hibernate.engine.NamedSQLQueryDefinition;
/*     */ import org.hibernate.engine.query.sql.NativeSQLQueryReturn;
/*     */ import org.hibernate.engine.query.sql.NativeSQLQueryRootReturn;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public abstract class QueryBinder
/*     */ {
/*  36 */   private static Log log = LogFactory.getLog(QueryBinder.class);
/*     */   
/*     */   public static void bindQuery(javax.persistence.NamedQuery queryAnn, ExtendedMappings mappings, boolean isDefault) {
/*  39 */     if (queryAnn == null) return;
/*  40 */     if (BinderHelper.isDefault(queryAnn.name())) {
/*  41 */       throw new AnnotationException("A named query must have a name when used in class or package level");
/*     */     }
/*     */     
/*  44 */     QueryHint[] hints = queryAnn.hints();
/*  45 */     String queryName = queryAnn.query();
/*  46 */     NamedQueryDefinition query = new NamedQueryDefinition(queryName, getBoolean(queryName, "org.hibernate.cacheable", hints), getString(queryName, "org.hibernate.cacheRegion", hints), getInteger(queryName, "org.hibernate.timeout", hints), getInteger(queryName, "org.hibernate.fetchSize", hints), getFlushMode(queryName, hints), getCacheMode(queryName, hints), getBoolean(queryName, "org.hibernate.readOnly", hints), getString(queryName, "org.hibernate.comment", hints), null);
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*  58 */     if (isDefault) {
/*  59 */       mappings.addDefaultQuery(queryAnn.name(), query);
/*     */     }
/*     */     else {
/*  62 */       mappings.addQuery(queryAnn.name(), query);
/*     */     }
/*  64 */     if (log.isInfoEnabled()) log.info("Binding Named query: " + queryAnn.name() + " => " + queryAnn.query());
/*     */   }
/*     */   
/*     */   public static void bindNativeQuery(javax.persistence.NamedNativeQuery queryAnn, ExtendedMappings mappings, boolean isDefault)
/*     */   {
/*  69 */     if (queryAnn == null) { return;
/*     */     }
/*  71 */     if (BinderHelper.isDefault(queryAnn.name())) {
/*  72 */       throw new AnnotationException("A named query must have a name when used in class or package level");
/*     */     }
/*     */     
/*  75 */     String resultSetMapping = queryAnn.resultSetMapping();
/*  76 */     QueryHint[] hints = queryAnn.hints();
/*  77 */     String queryName = queryAnn.query();
/*  78 */     NamedSQLQueryDefinition query; if (!BinderHelper.isDefault(resultSetMapping))
/*     */     {
/*  80 */       query = new NamedSQLQueryDefinition(queryName, resultSetMapping, null, getBoolean(queryName, "org.hibernate.cacheable", hints), getString(queryName, "org.hibernate.cacheRegion", hints), getInteger(queryName, "org.hibernate.timeout", hints), getInteger(queryName, "org.hibernate.fetchSize", hints), getFlushMode(queryName, hints), getCacheMode(queryName, hints), getBoolean(queryName, "org.hibernate.readOnly", hints), getString(queryName, "org.hibernate.comment", hints), null, getBoolean(queryName, "org.hibernate.callable", hints));
/*     */     }
/*     */     else
/*     */     {
/*     */       NamedSQLQueryDefinition query;
/*     */       
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*  96 */       if (!Void.TYPE.equals(queryAnn.resultClass()))
/*     */       {
/*     */ 
/*  99 */         NativeSQLQueryRootReturn entityQueryReturn = new NativeSQLQueryRootReturn("alias1", queryAnn.resultClass().getName(), new HashMap(), LockMode.READ);
/*     */         
/* 101 */         query = new NamedSQLQueryDefinition(queryName, new NativeSQLQueryReturn[] { entityQueryReturn }, null, getBoolean(queryName, "org.hibernate.cacheable", hints), getString(queryName, "org.hibernate.cacheRegion", hints), getInteger(queryName, "org.hibernate.timeout", hints), getInteger(queryName, "org.hibernate.fetchSize", hints), getFlushMode(queryName, hints), getCacheMode(queryName, hints), getBoolean(queryName, "org.hibernate.readOnly", hints), getString(queryName, "org.hibernate.comment", hints), null, getBoolean(queryName, "org.hibernate.callable", hints));
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */       }
/*     */       else
/*     */       {
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 118 */         throw new NotYetImplementedException("Pure native scalar queries are not yet supported"); } }
/*     */     NamedSQLQueryDefinition query;
/* 120 */     if (isDefault) {
/* 121 */       mappings.addDefaultSQLQuery(queryAnn.name(), query);
/*     */     }
/*     */     else {
/* 124 */       mappings.addSQLQuery(queryAnn.name(), query);
/*     */     }
/* 126 */     if (log.isInfoEnabled()) {
/* 127 */       log.info("Binding named native query: " + queryAnn.name() + " => " + queryAnn.query());
/*     */     }
/*     */   }
/*     */   
/*     */   public static void bindNativeQuery(org.hibernate.annotations.NamedNativeQuery queryAnn, ExtendedMappings mappings) {
/* 132 */     if (queryAnn == null) { return;
/*     */     }
/* 134 */     if (BinderHelper.isDefault(queryAnn.name())) {
/* 135 */       throw new AnnotationException("A named query must have a name when used in class or package level");
/*     */     }
/*     */     
/* 138 */     String resultSetMapping = queryAnn.resultSetMapping();
/* 139 */     NamedSQLQueryDefinition query; if (!BinderHelper.isDefault(resultSetMapping))
/*     */     {
/* 141 */       query = new NamedSQLQueryDefinition(queryAnn.query(), resultSetMapping, null, queryAnn.cacheable(), BinderHelper.isDefault(queryAnn.cacheRegion()) ? null : queryAnn.cacheRegion(), queryAnn.timeout() < 0 ? null : Integer.valueOf(queryAnn.timeout()), queryAnn.fetchSize() < 0 ? null : Integer.valueOf(queryAnn.fetchSize()), getFlushMode(queryAnn.flushMode()), getCacheMode(queryAnn.cacheMode()), queryAnn.readOnly(), BinderHelper.isDefault(queryAnn.comment()) ? null : queryAnn.comment(), null, queryAnn.callable());
/*     */     }
/*     */     else
/*     */     {
/*     */       NamedSQLQueryDefinition query;
/*     */       
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 157 */       if (!Void.TYPE.equals(queryAnn.resultClass()))
/*     */       {
/*     */ 
/* 160 */         NativeSQLQueryRootReturn entityQueryReturn = new NativeSQLQueryRootReturn("alias1", queryAnn.resultClass().getName(), new HashMap(), LockMode.READ);
/*     */         
/* 162 */         query = new NamedSQLQueryDefinition(queryAnn.query(), new NativeSQLQueryReturn[] { entityQueryReturn }, null, queryAnn.cacheable(), BinderHelper.isDefault(queryAnn.cacheRegion()) ? null : queryAnn.cacheRegion(), queryAnn.timeout() < 0 ? null : Integer.valueOf(queryAnn.timeout()), queryAnn.fetchSize() < 0 ? null : Integer.valueOf(queryAnn.fetchSize()), getFlushMode(queryAnn.flushMode()), getCacheMode(queryAnn.cacheMode()), queryAnn.readOnly(), BinderHelper.isDefault(queryAnn.comment()) ? null : queryAnn.comment(), null, queryAnn.callable());
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */       }
/*     */       else
/*     */       {
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 179 */         throw new NotYetImplementedException("Pure native scalar queries are not yet supported"); } }
/*     */     NamedSQLQueryDefinition query;
/* 181 */     mappings.addSQLQuery(queryAnn.name(), query);
/* 182 */     if (log.isInfoEnabled()) {
/* 183 */       log.info("Binding named native query: " + queryAnn.name() + " => " + queryAnn.query());
/*     */     }
/*     */   }
/*     */   
/*     */   public static void bindQueries(javax.persistence.NamedQueries queriesAnn, ExtendedMappings mappings, boolean isDefault) {
/* 188 */     if (queriesAnn == null) return;
/* 189 */     for (javax.persistence.NamedQuery q : queriesAnn.value()) {
/* 190 */       bindQuery(q, mappings, isDefault);
/*     */     }
/*     */   }
/*     */   
/*     */   public static void bindNativeQueries(javax.persistence.NamedNativeQueries queriesAnn, ExtendedMappings mappings, boolean isDefault) {
/* 195 */     if (queriesAnn == null) return;
/* 196 */     for (javax.persistence.NamedNativeQuery q : queriesAnn.value()) {
/* 197 */       bindNativeQuery(q, mappings, isDefault);
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */   public static void bindNativeQueries(org.hibernate.annotations.NamedNativeQueries queriesAnn, ExtendedMappings mappings)
/*     */   {
/* 204 */     if (queriesAnn == null) return;
/* 205 */     for (org.hibernate.annotations.NamedNativeQuery q : queriesAnn.value()) {
/* 206 */       bindNativeQuery(q, mappings);
/*     */     }
/*     */   }
/*     */   
/*     */   public static void bindQuery(org.hibernate.annotations.NamedQuery queryAnn, ExtendedMappings mappings) {
/* 211 */     if (queryAnn == null) return;
/* 212 */     if (BinderHelper.isDefault(queryAnn.name())) {
/* 213 */       throw new AnnotationException("A named query must have a name when used in class or package level");
/*     */     }
/*     */     
/*     */ 
/* 217 */     FlushMode flushMode = getFlushMode(queryAnn.flushMode());
/*     */     
/* 219 */     NamedQueryDefinition query = new NamedQueryDefinition(queryAnn.query(), queryAnn.cacheable(), BinderHelper.isDefault(queryAnn.cacheRegion()) ? null : queryAnn.cacheRegion(), queryAnn.timeout() < 0 ? null : Integer.valueOf(queryAnn.timeout()), queryAnn.fetchSize() < 0 ? null : Integer.valueOf(queryAnn.fetchSize()), flushMode, getCacheMode(queryAnn.cacheMode()), queryAnn.readOnly(), BinderHelper.isDefault(queryAnn.comment()) ? null : queryAnn.comment(), null);
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 232 */     mappings.addQuery(queryAnn.name(), query);
/* 233 */     if (log.isInfoEnabled()) log.info("Binding named query: " + queryAnn.name() + " => " + queryAnn.query());
/*     */   }
/*     */   
/*     */   private static FlushMode getFlushMode(FlushModeType flushModeType) {
/*     */     FlushMode flushMode;
/* 238 */     switch (flushModeType) {
/*     */     case ALWAYS: 
/* 240 */       flushMode = FlushMode.ALWAYS;
/* 241 */       break;
/*     */     case AUTO: 
/* 243 */       flushMode = FlushMode.AUTO;
/* 244 */       break;
/*     */     case COMMIT: 
/* 246 */       flushMode = FlushMode.COMMIT;
/* 247 */       break;
/*     */     case NEVER: 
/* 249 */       flushMode = FlushMode.MANUAL;
/* 250 */       break;
/*     */     case MANUAL: 
/* 252 */       flushMode = FlushMode.MANUAL;
/* 253 */       break;
/*     */     case PERSISTENCE_CONTEXT: 
/* 255 */       flushMode = null;
/* 256 */       break;
/*     */     default: 
/* 258 */       throw new AssertionFailure("Unknown flushModeType: " + flushModeType);
/*     */     }
/* 260 */     return flushMode;
/*     */   }
/*     */   
/*     */   private static CacheMode getCacheMode(CacheModeType cacheModeType) {
/* 264 */     switch (cacheModeType) {
/*     */     case GET: 
/* 266 */       return CacheMode.GET;
/*     */     case IGNORE: 
/* 268 */       return CacheMode.IGNORE;
/*     */     case NORMAL: 
/* 270 */       return CacheMode.NORMAL;
/*     */     case PUT: 
/* 272 */       return CacheMode.PUT;
/*     */     case REFRESH: 
/* 274 */       return CacheMode.REFRESH;
/*     */     }
/* 276 */     throw new AssertionFailure("Unknown cacheModeType: " + cacheModeType);
/*     */   }
/*     */   
/*     */ 
/*     */   public static void bindQueries(org.hibernate.annotations.NamedQueries queriesAnn, ExtendedMappings mappings)
/*     */   {
/* 282 */     if (queriesAnn == null) return;
/* 283 */     for (org.hibernate.annotations.NamedQuery q : queriesAnn.value()) {
/* 284 */       bindQuery(q, mappings);
/*     */     }
/*     */   }
/*     */   
/*     */   public static void bindSqlResultsetMappings(SqlResultSetMappings ann, ExtendedMappings mappings, boolean isDefault) {
/* 289 */     if (ann == null) return;
/* 290 */     for (SqlResultSetMapping rs : ann.value())
/*     */     {
/* 292 */       mappings.addSecondPass(new ResultsetMappingSecondPass(rs, mappings, true));
/*     */     }
/*     */   }
/*     */   
/*     */   public static void bindSqlResultsetMapping(SqlResultSetMapping ann, ExtendedMappings mappings, boolean isDefault)
/*     */   {
/* 298 */     mappings.addSecondPass(new ResultsetMappingSecondPass(ann, mappings, isDefault));
/*     */   }
/*     */   
/*     */   private static CacheMode getCacheMode(String query, QueryHint[] hints) {
/* 302 */     for (QueryHint hint : hints) {
/* 303 */       if ("org.hibernate.cacheMode".equals(hint.name())) {
/* 304 */         if (hint.value().equalsIgnoreCase(CacheMode.GET.toString())) {
/* 305 */           return CacheMode.GET;
/*     */         }
/* 307 */         if (hint.value().equalsIgnoreCase(CacheMode.IGNORE.toString())) {
/* 308 */           return CacheMode.IGNORE;
/*     */         }
/* 310 */         if (hint.value().equalsIgnoreCase(CacheMode.NORMAL.toString())) {
/* 311 */           return CacheMode.NORMAL;
/*     */         }
/* 313 */         if (hint.value().equalsIgnoreCase(CacheMode.PUT.toString())) {
/* 314 */           return CacheMode.PUT;
/*     */         }
/* 316 */         if (hint.value().equalsIgnoreCase(CacheMode.REFRESH.toString())) {
/* 317 */           return CacheMode.REFRESH;
/*     */         }
/*     */         
/* 320 */         throw new AnnotationException("Unknown CacheMode in hint: " + query + ":" + hint.name());
/*     */       }
/*     */     }
/*     */     
/* 324 */     return null;
/*     */   }
/*     */   
/*     */   private static FlushMode getFlushMode(String query, QueryHint[] hints) {
/* 328 */     for (QueryHint hint : hints) {
/* 329 */       if ("org.hibernate.flushMode".equals(hint.name())) {
/* 330 */         if (hint.value().equalsIgnoreCase(FlushMode.ALWAYS.toString())) {
/* 331 */           return FlushMode.ALWAYS;
/*     */         }
/* 333 */         if (hint.value().equalsIgnoreCase(FlushMode.AUTO.toString())) {
/* 334 */           return FlushMode.AUTO;
/*     */         }
/* 336 */         if (hint.value().equalsIgnoreCase(FlushMode.COMMIT.toString())) {
/* 337 */           return FlushMode.COMMIT;
/*     */         }
/* 339 */         if (hint.value().equalsIgnoreCase(FlushMode.NEVER.toString())) {
/* 340 */           return FlushMode.MANUAL;
/*     */         }
/* 342 */         if (hint.value().equalsIgnoreCase(FlushMode.MANUAL.toString())) {
/* 343 */           return FlushMode.MANUAL;
/*     */         }
/*     */         
/* 346 */         throw new AnnotationException("Unknown FlushMode in hint: " + query + ":" + hint.name());
/*     */       }
/*     */     }
/*     */     
/* 350 */     return null;
/*     */   }
/*     */   
/*     */   private static boolean getBoolean(String query, String hintName, QueryHint[] hints) {
/* 354 */     for (QueryHint hint : hints) {
/* 355 */       if (hintName.equals(hint.name())) {
/* 356 */         if (hint.value().equalsIgnoreCase("true")) {
/* 357 */           return true;
/*     */         }
/* 359 */         if (hint.value().equalsIgnoreCase("false")) {
/* 360 */           return false;
/*     */         }
/*     */         
/* 363 */         throw new AnnotationException("Not a boolean in hint: " + query + ":" + hint.name());
/*     */       }
/*     */     }
/*     */     
/* 367 */     return false;
/*     */   }
/*     */   
/*     */   private static String getString(String query, String hintName, QueryHint[] hints) {
/* 371 */     for (QueryHint hint : hints) {
/* 372 */       if (hintName.equals(hint.name())) {
/* 373 */         return hint.value();
/*     */       }
/*     */     }
/* 376 */     return null;
/*     */   }
/*     */   
/*     */   private static Integer getInteger(String query, String hintName, QueryHint[] hints) {
/* 380 */     for (QueryHint hint : hints) {
/* 381 */       if (hintName.equals(hint.name())) {
/*     */         try {
/* 383 */           return Integer.decode(hint.value());
/*     */         }
/*     */         catch (NumberFormatException nfe) {
/* 386 */           throw new AnnotationException("Not an integer in hint: " + query + ":" + hint.name(), nfe);
/*     */         }
/*     */       }
/*     */     }
/* 390 */     return null;
/*     */   }
/*     */ }


/* Location:              C:\Users\anandkum\Desktop\source\hibernate-annotations-3.3.1.GA.jar!\org\hibernate\cfg\annotations\QueryBinder.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */