/*     */ package org.hibernate.cfg.annotations.reflection;
/*     */ 
/*     */ import java.util.ArrayList;
/*     */ import java.util.HashMap;
/*     */ import java.util.List;
/*     */ import java.util.Map;
/*     */ import org.apache.commons.logging.Log;
/*     */ import org.apache.commons.logging.LogFactory;
/*     */ import org.dom4j.Document;
/*     */ import org.dom4j.Element;
/*     */ import org.hibernate.util.StringHelper;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public class XMLContext
/*     */ {
/*  19 */   private static Log log = LogFactory.getLog(XMLContext.class);
/*     */   
/*  21 */   public XMLContext() { this.classOverriding = new HashMap();
/*  22 */     this.defaultsOverriding = new HashMap();
/*  23 */     this.defaultElements = new ArrayList();
/*  24 */     this.defaultEntityListeners = new ArrayList();
/*  25 */     this.hasContext = false; }
/*     */   
/*     */   private Default globalDefaults;
/*     */   private Map<String, Element> classOverriding;
/*     */   private Map<String, Default> defaultsOverriding;
/*     */   
/*  31 */   public List<String> addDocument(Document doc) { this.hasContext = true;
/*  32 */     List<String> addedClasses = new ArrayList();
/*  33 */     Element root = doc.getRootElement();
/*     */     
/*  35 */     Element metadata = root.element("persistence-unit-metadata");
/*  36 */     if (metadata != null) {
/*  37 */       if (this.globalDefaults == null) {
/*  38 */         this.globalDefaults = new Default();
/*  39 */         this.globalDefaults.setMetadataComplete(metadata.element("xml-mapping-metadata-complete") != null ? Boolean.TRUE : null);
/*     */         
/*     */ 
/*     */ 
/*     */ 
/*  44 */         Element defaultElement = metadata.element("persistence-unit-defaults");
/*  45 */         if (defaultElement != null) {
/*  46 */           Element unitElement = defaultElement.element("schema");
/*  47 */           this.globalDefaults.setSchema(unitElement != null ? unitElement.getTextTrim() : null);
/*  48 */           unitElement = defaultElement.element("catalog");
/*  49 */           this.globalDefaults.setCatalog(unitElement != null ? unitElement.getTextTrim() : null);
/*  50 */           unitElement = defaultElement.element("access");
/*  51 */           this.globalDefaults.setAccess(unitElement != null ? unitElement.getTextTrim() : null);
/*  52 */           unitElement = defaultElement.element("cascade-persist");
/*  53 */           this.globalDefaults.setCascadePersist(unitElement != null ? Boolean.TRUE : null);
/*  54 */           this.defaultEntityListeners.addAll(addEntityListenerClasses(defaultElement, null, addedClasses));
/*     */         }
/*     */       }
/*     */       else {
/*  58 */         log.warn("Found more than one <persistence-unit-metadata>, subsequent ignored");
/*     */       }
/*     */     }
/*     */     
/*     */ 
/*  63 */     Default entityMappingDefault = new Default();
/*  64 */     Element unitElement = root.element("package");
/*  65 */     String packageName = unitElement != null ? unitElement.getTextTrim() : null;
/*  66 */     entityMappingDefault.setPackageName(packageName);
/*  67 */     unitElement = root.element("schema");
/*  68 */     entityMappingDefault.setSchema(unitElement != null ? unitElement.getTextTrim() : null);
/*  69 */     unitElement = root.element("catalog");
/*  70 */     entityMappingDefault.setCatalog(unitElement != null ? unitElement.getTextTrim() : null);
/*  71 */     unitElement = root.element("access");
/*  72 */     entityMappingDefault.setAccess(unitElement != null ? unitElement.getTextTrim() : null);
/*  73 */     this.defaultElements.add(root);
/*     */     
/*  75 */     List<Element> entities = root.elements("entity");
/*  76 */     addClass(entities, packageName, entityMappingDefault, addedClasses);
/*     */     
/*  78 */     entities = root.elements("mapped-superclass");
/*  79 */     addClass(entities, packageName, entityMappingDefault, addedClasses);
/*     */     
/*  81 */     entities = root.elements("embeddable");
/*  82 */     addClass(entities, packageName, entityMappingDefault, addedClasses);
/*  83 */     return addedClasses; }
/*     */   
/*     */   private List<Element> defaultElements;
/*     */   
/*  87 */   private void addClass(List<Element> entities, String packageName, Default defaults, List<String> addedClasses) { for (Element element : entities) {
/*  88 */       String className = buildSafeClassName(element.attributeValue("class"), packageName);
/*  89 */       if (this.classOverriding.containsKey(className))
/*     */       {
/*  91 */         throw new IllegalStateException("Duplicate XML entry for " + className);
/*     */       }
/*  93 */       addedClasses.add(className);
/*  94 */       this.classOverriding.put(className, element);
/*  95 */       Default localDefault = new Default();
/*  96 */       localDefault.override(defaults);
/*  97 */       String metadataCompleteString = element.attributeValue("metadata-complete");
/*  98 */       if (metadataCompleteString != null) {
/*  99 */         localDefault.setMetadataComplete(Boolean.valueOf(Boolean.parseBoolean(metadataCompleteString)));
/*     */       }
/* 101 */       String access = element.attributeValue("access");
/* 102 */       if (access != null) localDefault.setAccess(access);
/* 103 */       this.defaultsOverriding.put(className, localDefault);
/*     */       
/* 105 */       log.debug("Adding XML overriding information for " + className);
/* 106 */       addEntityListenerClasses(element, packageName, addedClasses);
/*     */     } }
/*     */   
/*     */   private List<String> defaultEntityListeners;
/*     */   private boolean hasContext;
/* 111 */   private List<String> addEntityListenerClasses(Element element, String packageName, List<String> addedClasses) { List<String> localAddedClasses = new ArrayList();
/* 112 */     Element listeners = element.element("entity-listeners");
/* 113 */     if (listeners != null) {
/* 114 */       List<Element> elements = listeners.elements("entity-listener");
/* 115 */       for (Element listener : elements) {
/* 116 */         String listenerClassName = buildSafeClassName(listener.attributeValue("class"), packageName);
/* 117 */         if (this.classOverriding.containsKey(listenerClassName))
/*     */         {
/* 119 */           if ("entity-listener".equals(((Element)this.classOverriding.get(listenerClassName)).getName())) {
/* 120 */             log.info("entity-listener duplication, first event definition will be used: " + listenerClassName);
/*     */ 
/*     */ 
/*     */           }
/*     */           else
/*     */           {
/*     */ 
/* 127 */             throw new IllegalStateException("Duplicate XML entry for " + listenerClassName);
/*     */           }
/*     */         } else {
/* 130 */           localAddedClasses.add(listenerClassName);
/* 131 */           this.classOverriding.put(listenerClassName, listener);
/*     */         }
/*     */       } }
/* 134 */     log.debug("Adding XML overriding information for listener: " + listeners);
/* 135 */     addedClasses.addAll(localAddedClasses);
/* 136 */     return localAddedClasses;
/*     */   }
/*     */   
/*     */   public static String buildSafeClassName(String className, String defaultPackageName) {
/* 140 */     if ((className.indexOf('.') < 0) && (StringHelper.isNotEmpty(defaultPackageName))) {
/* 141 */       className = StringHelper.qualify(defaultPackageName, className);
/*     */     }
/* 143 */     return className;
/*     */   }
/*     */   
/*     */   public static String buildSafeClassName(String className, Default defaults) {
/* 147 */     return buildSafeClassName(className, defaults.getPackageName());
/*     */   }
/*     */   
/*     */   public Default getDefault(String className) {
/* 151 */     Default xmlDefault = new Default();
/* 152 */     xmlDefault.override(this.globalDefaults);
/* 153 */     if (className != null) {
/* 154 */       Default entityMappingOverriding = (Default)this.defaultsOverriding.get(className);
/* 155 */       xmlDefault.override(entityMappingOverriding);
/*     */     }
/* 157 */     return xmlDefault;
/*     */   }
/*     */   
/*     */   public Element getXMLTree(String className, String methodName) {
/* 161 */     return (Element)this.classOverriding.get(className);
/*     */   }
/*     */   
/*     */   public List<Element> getAllDocuments() {
/* 165 */     return this.defaultElements;
/*     */   }
/*     */   
/*     */   public boolean hasContext() {
/* 169 */     return this.hasContext;
/*     */   }
/*     */   
/*     */   public static class Default {
/*     */     private String access;
/*     */     private String packageName;
/*     */     private String schema;
/*     */     private String catalog;
/*     */     private Boolean metadataComplete;
/*     */     private Boolean cascadePersist;
/*     */     
/*     */     public String getAccess() {
/* 181 */       return this.access;
/*     */     }
/*     */     
/*     */     protected void setAccess(String access) {
/* 185 */       if (("FIELD".equals(access)) || ("PROPERTY".equals(access))) {
/* 186 */         this.access = access.toLowerCase();
/*     */       }
/*     */       else {
/* 189 */         this.access = access;
/*     */       }
/*     */     }
/*     */     
/*     */     public String getCatalog() {
/* 194 */       return this.catalog;
/*     */     }
/*     */     
/*     */     protected void setCatalog(String catalog) {
/* 198 */       this.catalog = catalog;
/*     */     }
/*     */     
/*     */     public String getPackageName() {
/* 202 */       return this.packageName;
/*     */     }
/*     */     
/*     */     protected void setPackageName(String packageName) {
/* 206 */       this.packageName = packageName;
/*     */     }
/*     */     
/*     */     public String getSchema() {
/* 210 */       return this.schema;
/*     */     }
/*     */     
/*     */     protected void setSchema(String schema) {
/* 214 */       this.schema = schema;
/*     */     }
/*     */     
/*     */     public Boolean getMetadataComplete() {
/* 218 */       return this.metadataComplete;
/*     */     }
/*     */     
/*     */     public boolean canUseJavaAnnotations() {
/* 222 */       return (this.metadataComplete == null) || (!this.metadataComplete.booleanValue());
/*     */     }
/*     */     
/*     */     protected void setMetadataComplete(Boolean metadataComplete) {
/* 226 */       this.metadataComplete = metadataComplete;
/*     */     }
/*     */     
/*     */     public Boolean getCascadePersist() {
/* 230 */       return this.cascadePersist;
/*     */     }
/*     */     
/*     */     void setCascadePersist(Boolean cascadePersist) {
/* 234 */       this.cascadePersist = cascadePersist;
/*     */     }
/*     */     
/*     */     public void override(Default globalDefault) {
/* 238 */       if (globalDefault != null) {
/* 239 */         if (globalDefault.getAccess() != null) this.access = globalDefault.getAccess();
/* 240 */         if (globalDefault.getPackageName() != null) this.packageName = globalDefault.getPackageName();
/* 241 */         if (globalDefault.getSchema() != null) this.schema = globalDefault.getSchema();
/* 242 */         if (globalDefault.getCatalog() != null) this.catalog = globalDefault.getCatalog();
/* 243 */         if (globalDefault.getMetadataComplete() != null) {
/* 244 */           this.metadataComplete = globalDefault.getMetadataComplete();
/*     */         }
/*     */         
/* 247 */         if (globalDefault.getCascadePersist() != null) this.cascadePersist = globalDefault.getCascadePersist();
/*     */       }
/*     */     }
/*     */   }
/*     */   
/*     */   public List<String> getDefaultEntityListeners() {
/* 253 */     return this.defaultEntityListeners;
/*     */   }
/*     */ }


/* Location:              C:\Users\anandkum\Desktop\source\hibernate-annotations-3.3.1.GA.jar!\org\hibernate\cfg\annotations\reflection\XMLContext.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */