/*     */ package org.hibernate.ejb.transaction;
/*     */ 
/*     */ import javax.transaction.SystemException;
/*     */ import javax.transaction.Transaction;
/*     */ import javax.transaction.TransactionManager;
/*     */ import org.hibernate.HibernateException;
/*     */ import org.hibernate.TransactionException;
/*     */ import org.hibernate.engine.SessionFactoryImplementor;
/*     */ import org.hibernate.jdbc.JDBCContext;
/*     */ import org.hibernate.transaction.CMTTransaction;
/*     */ import org.hibernate.transaction.TransactionFactory.Context;
/*     */ import org.hibernate.util.JTAHelper;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public class JoinableCMTTransaction
/*     */   extends CMTTransaction
/*     */ {
/*     */   private JoinStatus status;
/*     */   
/*     */   public JoinableCMTTransaction(JDBCContext jdbcContext, TransactionFactory.Context transactionContext)
/*     */   {
/*  24 */     super(jdbcContext, transactionContext);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   public boolean isTransactionInProgress(JDBCContext jdbcContext, TransactionFactory.Context transactionContext)
/*     */   {
/*     */     try
/*     */     {
/*  34 */       return (this.status == JoinStatus.JOINED) && (isTransactionInProgress(transactionContext.getFactory().getTransactionManager().getTransaction()));
/*     */ 
/*     */     }
/*     */     catch (SystemException se)
/*     */     {
/*  39 */       throw new TransactionException("Unable to check transaction status", se);
/*     */     }
/*     */   }
/*     */   
/*     */   private boolean isTransactionInProgress() {
/*     */     try {
/*  45 */       Transaction transaction = this.transactionContext.getFactory().getTransactionManager().getTransaction();
/*  46 */       return isTransactionInProgress(transaction);
/*     */     }
/*     */     catch (SystemException se) {
/*  49 */       throw new TransactionException("Unable to check transaction status", se);
/*     */     }
/*     */   }
/*     */   
/*     */   private boolean isTransactionInProgress(Transaction tx) throws SystemException {
/*  54 */     return (JTAHelper.isTransactionInProgress(tx)) && (!JTAHelper.isRollback(tx.getStatus()));
/*     */   }
/*     */   
/*     */   void tryJoiningTransaction() {
/*  58 */     if (this.status == JoinStatus.MARKED_FOR_JOINED) {
/*  59 */       if (isTransactionInProgress()) {
/*  60 */         this.status = JoinStatus.JOINED;
/*     */       }
/*     */       else {
/*  63 */         this.status = JoinStatus.NOT_JOINED;
/*     */       }
/*     */     }
/*     */   }
/*     */   
/*     */   public void begin() throws HibernateException
/*     */   {
/*  70 */     super.begin();
/*  71 */     this.status = JoinStatus.JOINED;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public void commit()
/*     */     throws HibernateException
/*     */   {
/*  81 */     super.commit();
/*     */   }
/*     */   
/*     */   public JoinStatus getStatus()
/*     */   {
/*  86 */     return this.status;
/*     */   }
/*     */   
/*     */   public void resetStatus() {
/*  90 */     this.status = JoinStatus.NOT_JOINED;
/*     */   }
/*     */   
/*     */   public void markForJoined() {
/*  94 */     if (this.status != JoinStatus.JOINED) this.status = JoinStatus.MARKED_FOR_JOINED;
/*     */   }
/*     */   
/*     */   public static enum JoinStatus {
/*  98 */     NOT_JOINED, 
/*  99 */     MARKED_FOR_JOINED, 
/* 100 */     JOINED;
/*     */     
/*     */     private JoinStatus() {}
/*     */   }
/*     */ }


/* Location:              C:\Users\anandkum\Desktop\source\hibernate-entitymanager-3.3.2.GA.jar!\org\hibernate\ejb\transaction\JoinableCMTTransaction.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */